module ExampleAC where

import Terminals2Silver
import Source2Silver
import View2Silver
import BidiEngine
import Get2Silver
import Put2Silver
import Types

t = invert transfCA

main :: IO()
main = do putStrLn "Generating Silver Files..."
          writeFile "Gram/Concrete_Generated.sv" (source2Silver concrete_grammar)
          writeFile "Gram/Abstract_Generated.sv" (view2Silver   (abstract_grammar, concrete_grammar))
          writeFile "Gram/Get_Generated.sv"      (get2Silver    transfCA)
          writeFile "Gram/Put_Generated.sv"      (put2Silver   (invert transfCA) (extractCanBes transfCA) (cfgNonTerms concrete_grammar))
          writeFile "Gram/Terminals.sv"          (initialize)
          putStrLn ("Silver Files generated in './Gram'.")

transfCA :: Transf Abstract_Terminals Abstract_Nonterminals Abstract_Prod_Names Concrete_Terminals Concrete_Nonterminals Concrete_Prod_Names
transfCA = Transf [ ( get E A (Pattern Add   E [P_Arg 'a' E, P_Term Plus,  P_Arg 'b' T])    ,    Pattern ADDa A [get E A (P_Arg 'a' E), get T A (P_Arg 'b' T )] )
                  , ( get E A (Pattern Sub   E [P_Arg 'a' E, P_Term Minus, P_Arg 'b' T])    ,    Pattern SUBa A [get E A (P_Arg 'a' E), get T A (P_Arg 'b' T )] )
                  , ( get E A (Pattern Et    E [P_Arg 'a' T]                           )    ,    get E A (P_Arg 'a' T)                                          )
                  , ( get T A (Pattern Mul   T [P_Arg 'a' T, P_Term Ex,    P_Arg 'b' F])    ,    Pattern MULa A [get T A (P_Arg 'a' T), get F A (P_Arg 'b' F )] )
                  , ( get T A (Pattern Tf    T [P_Arg 'a' F]                           )    ,    get F A (P_Arg 'a' F)                                          )
                  , ( get F A (Pattern Neg   F [P_Term Minus, P_Arg 'a' F]             )    ,    Pattern SUBa A [Pattern VALUEa A [P_Lexemme 'i' "0" Int_Lit], get F A (P_Arg 'a' F)] )
                  , ( get F A (Pattern Nest  F [P_Term LeftParen, P_Arg 'a' E, P_Term RightParen])    ,    get F A (P_Arg 'a' E) )
                  , ( get F A (Pattern Value F [P_Prim 'a' Int_Lit ]                   )    ,    Pattern VALUEa A [P_Prim 'a' Int_Lit])
                  ]

-- -- -- Concrete Grammar
data Concrete_Terminals = LeftParen | RightParen | Plus | Minus | Ex
                        deriving (Show, Eq, Ord)
instance RegExp Concrete_Terminals where
  regExp LeftParen  = "("
  regExp RightParen = ")"
  regExp Plus       = "+"
  regExp Minus      = "-"
  regExp Ex         = "*"

data Concrete_Nonterminals = E | T | F
                           deriving (Show, Eq, Ord)
data Concrete_Prod_Names = Add | Sub | Et | Mul | Tf | Neg | Nest | Value 
                         deriving (Show, Eq, Ord)

concrete_grammar :: CFG Concrete_Terminals Concrete_Nonterminals Concrete_Prod_Names
concrete_grammar = CFG [E, T, F] [Int_Lit] [LeftParen, RightParen, Plus, Minus, Ex] [Add, Sub, Et, Mul, Tf, Neg, Nest, Value]
                       [ Prod Add   E [NonTerm E,      Term Plus,  NonTerm T      ]
                       , Prod Sub   E [NonTerm E,      Term Minus, NonTerm T      ]
                       , Prod Et    E [NonTerm T                                  ]
                       , Prod Mul   T [NonTerm T,      Term Ex,    NonTerm F      ]
                       , Prod Tf    T [NonTerm F                                  ]
                       , Prod Neg   F [Term Minus,     NonTerm F                  ]
                       , Prod Nest  F [Term LeftParen, NonTerm E, Term RightParen ]
                       , Prod Value F [Prim Int_Lit                               ]
                       ]

-- -- -- Abstract Grammar
data Abstract_Terminals = Abstract_Nonterminals
                          deriving (Show, Ord, Eq)
data Abstract_Nonterminals = A
                           deriving (Show, Eq, Ord)
instance RegExp Abstract_Terminals where
  regExp _ = ""

data Abstract_Prod_Names = ADDa | SUBa | MULa | VALUEa | ZERO
                         deriving (Show, Eq, Ord)

abstract_grammar :: CFG Abstract_Terminals Abstract_Nonterminals Abstract_Prod_Names
abstract_grammar = CFG [A] [Int_Lit] [] [ADDa, SUBa, MULa, VALUEa]
                       [ Prod ADDa   A [NonTerm A, NonTerm A]
                       , Prod SUBa   A [NonTerm A, NonTerm A]
                       , Prod MULa   A [NonTerm A, NonTerm A]
                       , Prod VALUEa A [Prim Int_Lit        ]
                       ]
