module ExampleDecls where

import Terminals2Silver
import Source2Silver
import View2Silver
import BidiEngine
import Get2Silver
import Put2Silver
import Types

t = invert transfCA

main :: IO()
main = do putStrLn "Generating Silver Files..."
          writeFile "Gram/Concrete_Generated.sv" (source2Silver concrete_grammar)
          writeFile "Gram/Abstract_Generated.sv" (view2Silver   (abstract_grammar, concrete_grammar))
          writeFile "Gram/Get_Generated.sv"      (get2Silver    transfCA)
          writeFile "Gram/Put_Generated.sv"      (put2Silver   (invert transfCA) (extractCanBes transfCA) (cfgNonTerms concrete_grammar))
          writeFile "Gram/Terminals.sv"          (initialize)
          putStrLn ("Silver Files generated in './Gram'.")

transfCA :: Transf Abstract_Terminals Abstract_NonTerminals Abstract_ProdNames Concrete_Terminals Concrete_NonTerminals Concrete_ProdNames
transfCA = Transf [ -- get_Root_on_Root_c ( root_c(ds) )  ->  root ( get_Decl_on_Decls_c(ds) )
                    ( get Root_c  Root  (Pattern P_root_c     Root_c  [P_Arg 'a' Decls_c]) ,
                      Pattern P_root    Root [get Decls_c Decl (P_Arg 'a' Decls_c)]        )
       
                    -- get_Decl_on_Decls_c ( consDecl_c( d, rest ) ) -> seqDecl( get_Decl_on_Decl_c(d), get_Decl_on_Decls_c(rest) )
                  , ( get Decls_c Decl (Pattern P_consDecl_c Decls_c [P_Arg 'a' Decl_c, P_Arg 'b' Decls_c])              ,
                      Pattern P_seqDecl Decl [get Decl_c Decl (P_Arg 'a' Decl_c), get Decls_c Decl (P_Arg 'b' Decls_c)] )
       
                    -- get_Decl_on_Decls_c ( nilDecl_c() )  ->  skipDecl()
                  , ( get Decls_c Decl (Pattern P_nilDecl_c Decls_c [])    ,    Pattern P_skipDecl Decl [] )
       
                    -- get_Decl_on_Decl_c ( multiDecl_c( t, oneVar_c( var_t(lex) ), semi_t()) ) -> varDecl( get_Type_on_Type_c(t), lex )
                  , ( get Decl_c Decl (Pattern P_multiDecl_c Decl_c [P_Arg 't' Type_c, Pattern P_oneVar_c Vars_c [P_Prim 'v' String_Lit], P_Term Semi_t ]) ,
                      Pattern P_varDecl Decl [get Type_c Type (P_Arg 't' Type_c), P_Prim 'v' String_Lit]                                                                   )
       
                    -- get_Decl_on_Decl_c ( multiDecl_c( t, consVar_c( var_t(lex), comma_t(), rest), semi_t() ) )
                    -- ->  seqDecl( varDecl( get_Type_on_Type_c(t), lex),  ( get_Decl_on_Decl_c (multiDecl_c( t, rest, semi_t()) ) ) )
                  , ( get Decl_c Decl (Pattern P_multiDecl_c Decl_c [P_Arg 't' Type_c, Pattern P_consVar_c Vars_c [P_Prim 'v' String_Lit, P_Term Comma_t, P_Arg 'a' Vars_c], P_Term Semi_t ])                                         ,
                      Pattern P_seqDecl Decl [Pattern P_varDecl Decl [get Type_c Type (P_Arg 't' Type_c), P_Prim 'v' String_Lit], get Decl_c Decl (Pattern P_multiDecl_c Decl_c [P_Arg 't' Type_c,P_Arg 'a' Vars_c , P_Term Semi_t])] )
       
                  , ( get Type_c Type (Pattern P_intType_c   Type_c [P_Term Int_t])      ,    Pattern P_intType   Type [] )
                  , ( get Type_c Type (Pattern P_floatType_c Type_c [P_Term Float_t])    ,    Pattern P_floatType Type [] )
                  , ( get Type_c Type (Pattern P_arrayType_c Type_c [P_Term Array_t])    ,    Pattern P_arrayType Type [] )
                  ]

-- -- Abstract Grammar
data Abstract_Terminals = Empty_Terminals
  deriving (Show, Eq, Ord)

instance RegExp Abstract_Terminals where
  regExp = undefined

data Abstract_NonTerminals = Root | Decl | Type
                        deriving (Show, Eq, Ord)

data Abstract_ProdNames = P_root | P_seqDecl | P_skipDecl | P_varDecl | P_intType | P_floatType | P_arrayType
                          deriving (Show, Eq, Ord)

abstract_grammar :: CFG Abstract_Terminals Abstract_NonTerminals Abstract_ProdNames
abstract_grammar = CFG [Root, Decl, Type] [String_Lit] []
                       [P_root, P_seqDecl, P_skipDecl, P_varDecl, P_intType, P_floatType, P_arrayType]
                       [ Prod P_root Root      [NonTerm Decl]
                       , Prod P_seqDecl  Decl  [NonTerm Decl, NonTerm Decl]
                       , Prod P_skipDecl Decl  []
                       , Prod P_varDecl Decl   [NonTerm Type, Prim String_Lit]
                       , Prod P_intType Type   []
                       , Prod P_floatType Type []
                       , Prod P_arrayType Type []
                       ]

-- -- Concrete Grammar
data Concrete_Terminals = Int_t | Float_t | Array_t | Comma_t | Semi_t
                        deriving (Show, Eq, Ord)

instance RegExp Concrete_Terminals where
  regExp Int_t   = "int"
  regExp Float_t = "float"
  regExp Array_t = "array"
  regExp Comma_t = ","
  regExp Semi_t  = ";"

data Concrete_NonTerminals = Root_c | Decls_c | Decl_c | Vars_c | Type_c 
                        deriving (Show, Eq, Ord)

data Concrete_ProdNames = P_root_c | P_consDecl_c | P_nilDecl_c | P_multiDecl_c | P_consVar_c | P_oneVar_c | P_intType_c | P_floatType_c | P_arrayType_c
                         deriving (Show, Eq, Ord)

concrete_grammar :: CFG Concrete_Terminals Concrete_NonTerminals Concrete_ProdNames
concrete_grammar = CFG [Root_c, Decls_c, Decl_c, Vars_c, Type_c] [String_Lit]
                       [Int_t, Float_t, Array_t, Comma_t, Semi_t]
                       [P_root_c, P_consDecl_c, P_nilDecl_c, P_multiDecl_c, P_consVar_c, P_oneVar_c, P_intType_c, P_floatType_c, P_arrayType_c]
                       [ Prod P_root_c      Root_c  [NonTerm Decls_c]
                       , Prod P_consDecl_c  Decls_c [NonTerm Decl_c, NonTerm Decls_c]
                       , Prod P_nilDecl_c   Decls_c []
                       , Prod P_multiDecl_c Decl_c  [NonTerm Type_c, NonTerm Vars_c, Term Semi_t] 
                       , Prod P_consVar_c   Vars_c  [Prim String_Lit, Term Comma_t, NonTerm Vars_c]
                       , Prod P_oneVar_c    Vars_c  [Prim String_Lit]
                       , Prod P_intType_c   Type_c  [Term Int_t]
                       , Prod P_floatType_c Type_c  [Term Float_t]
                       , Prod P_arrayType_c Type_c  [Term Array_t]
                       ]
