grammar Gram ;

attribute pp occurs on Root, Decl, Type ; -- Decls,
attribute show occurs on Root, Decl, Type ;  -- Decls,

synthesized attribute put_Root_c  :: Root_c  occurs on Root ;
synthesized attribute put_Decls_c :: Decls_c occurs on Decl ;
synthesized attribute put_Decl_c  :: Decl_c  occurs on Decl ;
synthesized attribute put_Type_c  :: Type_c  occurs on Type ;

synthesized attribute equals :: Boolean ;
inherited attribute equals_in<a> :: a ;

-- Root
nonterminal Root ;

synthesized attribute repair :: Root occurs on Root ;

abstract production root
r::Root ::= d::Decl
{ r.pp = d.pp ;
  r.show = "root( " ++ d.show ++ " )";

  -- The get rule was:
  --   get_Root ( root_c(ds) )  ->  root ( get_Decl(ds) )
  -- Simple inversion of the rewrite rule follows below:
  --   put_Root_c ( root (d) ) ->  root_c( put_Decls_c(d) ) 
  --     where d == get_Decl(ds).
  r.put_Root_c = case r of 
                    root(d) -> root_c( (if d.needs_repair_Decl_for_Decls_c
                                        then d.repair_Decl.put_Decls_c
                                        else d.put_Decls_c) ) 
                 end ;

  d.repair_Decl_tail = skipDecl() ;


  -- Only need this rule to show the repaired tree, it should not be required
  -- for real use.
  r.repair = root ( d.repair_Decl ) ;

  r.equals = case r.equals_in of
             | root (d2) -> equals_Decl(d,d2)
             | _ -> false 
             end ;             
}
function equals_Root
Boolean ::= t1::Root t2::Root
{ return t1.equals ;
  t1.equals_in = t2 ;
}
attribute equals occurs on Root ;
attribute equals_in<Root> occurs on Root ;

-- Decl
nonterminal Decl ;

synthesized attribute repair_Decl :: Decl occurs on Decl ;
synthesized attribute needs_repair_Decl_for_Decls_c :: Boolean occurs on Decl ;
synthesized attribute needs_repair_Decl_for_Decl_c :: Boolean occurs on Decl ;
inherited attribute repair_Decl_tail :: Decl occurs on Decl ;

abstract production seqDecl
ds::Decl ::= d1::Decl d2::Decl
{ ds.pp = d1.pp ++ "\n" ++ d2.pp ;
  ds.show = "seqDecl( " ++ d1.show ++ ", " ++ d2.show ++ " )";

  -- The get rules are:
  --   get_Decl ( consDecl_c( d, rest ) )  
  --     ->  seqDecl( get_Decl(d), get_Decl(rest) )
  -- and
  --   get_Decl ( multiDecl_c( t, consVar_c( var_t(lexeme), comma_t(), rest), semi_t() ) )
  --     ->  seqDecl( varDecl( get(t), lexeme), 
  --                  ( get_Decl (multiDecl_c( t, rest, semi_t()) ) ) )

  -- Inverting the first rule gives:
  --   put_Decls_c ( seqDecl( d1, d2 ) )
  --     ->  consDecl_c( put_Decl_c(d), put_Decls_c(rest) ) 
  ds.put_Decls_c = consDecl_c( d1.put_Decl_c, d2.put_Decls_c ) ;

  -- assuming that the above equation is correct, then we can always 'put' to
  -- a Decls_c
  ds.needs_repair_Decl_for_Decls_c = d1.needs_repair_Decl_for_Decl_c ||
                                     d2.needs_repair_Decl_for_Decls_c ;

  -- Original second rule:
  --   get_Decl ( multiDecl_c( t, consVar_c( var_t(lexeme), comma_t(), rest), semi_t() ) )
  --     ->  seqDecl( varDecl( get(t), lexeme), 
  --                  ( get_Decl (multiDecl_c( t, rest, semi_t()) ) ) )
  -- Inverting the second rule gives:
  --   put_Decl ( seqDecl( varDecl(t, lexeme), 
  --                     ( d == get_Decl (multiDecl_c( t, rest, semi_t()) ) ) )
  --     ->  multiDecl_c( put_Type(t), consVar_c( var_t(lexeme), comma_t(), rest), semi_t() ) )


  ds.put_Decl_c = 
    case ds of
    | seqDecl ( varDecl (t, lex), d )
      -> case d.put_Decl_c of
         | multiDecl_c ( t2, rest, semi )
           -> if equals_Type (t2.get_Type, t)
              then multiDecl_c ( t.put_Type_c, consVar_c ( terminal (Var_t, lex), 
                                                           terminal (Comma_t, ","),
                                                           rest) , semi )
              else error ("seqDecl cannot put to a Decl_c 1") 
         | _ -> error ("seqDecl cannot put to a Decl_c 2") 
         end
         | _ -> error ("seqDecl cannot put to a Decl_c 3") 
    end ;

  -- to determine if this can 'put' to a Decl, we have the same
  -- structure as above, replacing trees with false (or maybe
  -- disjuctions??) error expressions with true
  ds.needs_repair_Decl_for_Decl_c = 
    case ds of
    | seqDecl ( varDecl (t, lex),
                d )
      -> case d.put_Decl_c of
         | multiDecl_c ( t2, rest, semi )
           -> if equals_Type (t2.get_Type, t)
              then false
              else true
         | _ -> true
         end
         | _ -> true
    end ;

  ds.repair_Decl = d1.repair_Decl ;
  d1.repair_Decl_tail = d2.repair_Decl ;
  d2.repair_Decl_tail = ds.repair_Decl_tail ;

  ds.equals = case ds.equals_in of
             | seqDecl (d1x, d2x) -> equals_Decl(d1,d1x) && equals_Decl(d2,d2x)
             | _ -> false 
             end ;             
}

abstract production skipDecl
ds::Decl ::=
{ ds.pp = "" ;
  ds.show = "skipDecl()";

  ds.put_Decls_c = nilDecl_c() ;
  ds.needs_repair_Decl_for_Decls_c = false ;

  ds.put_Decl_c  = error ("skipDecl cannot put to a Decl_c") ;
  ds.needs_repair_Decl_for_Decl_c = true ;

  ds.repair_Decl = ds.repair_Decl_tail ;

  ds.equals = case ds.equals_in of
              | skipDecl() -> true
              | _ -> false
              end ;
}


abstract production varDecl
d::Decl ::= t::Type v::String
{ d.pp = t.pp ++ " " ++ v ++ " ;" ;
  d.show = "varDecl( " ++ t.show ++ ", " ++ v ++ " )";

  -- The get rule is:
  --   get_Decl ( multiDecl_c( t, oneVar_c( var_t(lexeme) ), semi_t()) )
  --     ->  varDecl( get_Type(t), lexeme )
  -- The inversion is:
  --   put_Decl_c ( varDecl( t, lex ) )
  --     ->  multiDecl_c( put_Type_c(t), oneVar_c(var_t(lex)), semi_t() )
  d.put_Decl_c = 
    case d of
      varDecl(t, v) -> multiDecl_c( t.put_Type_c,
                                    oneVar_c( terminal(Var_t, v) ), 
                                    terminal(Semi_t,";") ) 
    end ;
  d.needs_repair_Decl_for_Decl_c = false ;

  d.put_Decls_c = error ("... not sure if we can do this, define put_Decls on varDecl ...") ;
  d.needs_repair_Decl_for_Decls_c = true ;

  d.repair_Decl = seqDecl (d, d.repair_Decl_tail ) ;

  d.equals = case d.equals_in of
             | varDecl( t2, v2) -> equals_Type(t, t2) && v == v2 
             | _ -> false 
             end ;             
}

function equals_Decl
Boolean ::= t1::Decl t2::Decl
{ return t1.equals ;
  t1.equals_in = t2 ;
}
attribute equals occurs on Decl ;
attribute equals_in<Decl> occurs on Decl ;


-- Type
nonterminal Type ;

abstract production intType
t::Type ::= 
{ t.pp = "int" ;
  t.show = "intType()";

  t.put_Type_c = intType_c( terminal(Int_t, "int") ) ;

  t.equals = case t.equals_in of
             | intType() -> true
             | _ -> false 
             end ;             
}

abstract production floatType
t::Type ::= 
{ t.pp = "float" ;
  t.show = "floatType()";

  t.put_Type_c = floatType_c( terminal(Float_t, "float") ) ;

  t.equals = case t.equals_in of
             | intType() -> true
             | _ -> false 
             end ;             
}

abstract production arrayType
t::Type ::= et::Type
{ t.pp = "array " ++ et.pp ;
  t.show = "arrayType()";

  t.put_Type_c = arrayType_c( terminal(Array_t, "array"), et.put_Type_c ) ;

  t.equals = case t.equals_in of
             | arrayType(et2) -> equals_Type(et, et2)
             | _ -> false 
             end ;             
}

function equals_Type
Boolean ::= t1::Type t2::Type
{ return t1.equals ;
  t1.equals_in = t2 ;
}
attribute equals occurs on Type ;
attribute equals_in<Type> occurs on Type ;


