grammar Gram ;

synthesized attribute pp :: String
  occurs on Root_c, Decls_c, Decl_c, Vars_c, Type_c ;

synthesized attribute show :: String
  occurs on Root_c, Decls_c, Decl_c, Vars_c, Type_c ;

synthesized attribute get_Root_on_Root_c  :: Root occurs on Root_c ;
synthesized attribute get_Decl_on_Decls_c :: Decl occurs on Decls_c ;
synthesized attribute get_Decl_on_Decl_c  :: Decl occurs on Decl_c ;
synthesized attribute get_Type_on_Type_c  :: Type occurs on Type_c ;

-- Root_c
nonterminal Root_c ;
concrete production root_c
r::Root_c ::= ds::Decls_c
{ r.pp = ds.pp ;
  r.show = "root_c ( " ++ ds.show ++ " )";

  r.get_Root_on_Root_c = 
    case r of
    --- get_Root_on_Root_c ( root_c(ds) )  ->  root ( get_Decl_on_Decls_c(ds) )
    | root_c(ds) -> root(ds.get_Decl_on_Decls_c) 
    end ;
}

-- Decls_c
nonterminal Decls_c ;
concrete production consDecl_c
ds::Decls_c ::= d::Decl_c rest::Decls_c
{ ds.pp = d.pp ++ "\n" ++ rest.pp ;
  ds.show = "consDecl_c(" ++ d.show ++ ", " ++ rest.show ++ ")";

  ds.get_Decl_on_Decls_c = case ds of
  --- get_Decl_on_Decls_c ( consDecl_c( d, rest ) )  
  ---   ->  seqDecl( get_Decl_on_Decl_c(d), get_Decl_on_Decls_c(rest) )
  | consDecl_c(d, rest) 
    -> seqDecl(d.get_Decl_on_Decl_c, rest.get_Decl_on_Decls_c) 
  end ;
}

concrete production nilDecl_c
ds::Decls_c ::=
{ ds.pp = "" ;
  ds.show = "nilDecl_c";

  ds.get_Decl_on_Decls_c = case ds of
  --- get_Decl_on_Decls_c ( nilDecl_c() )  ->  skipDecl()
  | nilDecl_c() -> skipDecl()
  end ;
}

-- Decl_c
nonterminal Decl_c ;

concrete production multiDecl_c
d::Decl_c ::= t::Type_c vs::Vars_c s::';'
{ d.pp = t.pp ++ " " ++ vs.pp ++ " ;" ;
  d.show = "multiDecl_c( " ++ t.show ++ ", " ++ vs.show ++ ", " ++ "';'" ++ " )";

  d.get_Decl_on_Decls_c = case d of
  --- get_Decl_on_Decl_c ( multiDecl_c( t, oneVar_c( var_t(lex) ), semi_t()) )
  ---   ->  varDecl( get_Type_on_Type_c(t), lex )
  | multiDecl_c( t, oneVar_c(v), _)
      ->  varDecl( t.get_Type_on_Type_c, v.lexeme )

  --- get_Decl_on_Decl_c ( multiDecl_c( t, consVar_c( var_t(lex), comma_t(), rest), semi_t() ) )
  ---   ->  seqDecl( varDecl( get_Type_on_Type_c(t), lex), 
  ---                ( get_Decl_on_Decl_c (multiDecl_c( t, rest, semi_t()) ) ) )
   | multiDecl_c( t, consVar_c(v, comma, rest), semi)
     -> seqDecl( varDecl(t.get_Type_on_Type_c, v.lexeme), 
                 ( multiDecl_c(t, rest, terminal(Semi_t,";") ).get_Decl_on_Decl_c) ) 
   | _ -> error("get failed to match in multiDecl_c\n")
   end ;
}

-- Vars_c
nonterminal Vars_c ;

{- Vars_c do not map to anything in the abstract syntax.  Our analysis
   of the rules should see that this is OK.
 -}
concrete production consVar_c
vs::Vars_c ::= v::Var_t ',' rest::Vars_c
{ vs.pp = v.lexeme ++ ", " ++ rest.pp ;
  vs.show = "consVar_c( " ++ v.lexeme ++ ", " ++ "','" ++ ", " ++ rest.show ++ " )";
}

concrete production oneVar_c
vs::Vars_c ::= v::Var_t
{ vs.pp = v.lexeme ;
  vs.show = "oneVar_c( " ++ v.lexeme ++ " )";
}

-- Type_c
nonterminal Type_c ;

concrete production intType_c
t::Type_c ::= 'int'
{ t.pp = "int" ;
  t.show = "intType_c()";

  t.get_Type_on_Type_c = case t of
  --- get_Type_on_Type_c ( intType_c(_) ) -> intType()
  | intType_c(_) -> intType()
  end ;
  -- this is, of course redundant.  
  -- t.get = intType()   would be just fine.

  t.equals = case t.equals_in of
             | intType_c(_) -> true
             | _ -> false 
             end ;             
}

concrete production floatType_c
t::Type_c ::= 'float'
{ t.pp = "float" ;
  t.show = "floatType_c()";

  --- get_Type_on_Type_c ( floatType_c(_) ) -> floatType()
  t.get_Type_on_Type_c = floatType() ;

  t.equals = case t.equals_in of
             | floatType_c(_) -> true
             | _ -> false 
             end ;
}

concrete production arrayType_c
t::Type_c ::= 'array' et::Type_c
{ t.pp = "array " ++ et.pp ;
  t.show = "arrayType_c()";

  --- get_Type_on_Type_c ( arrayType_c(_,et) ) -> arrayType( get_Type_on_Type_c(et) )
  t.get_Type_on_Type_c = arrayType(et.get_Type_on_Type_c) ;

  t.equals = case t.equals_in of
             | arrayType_c(_,et2) -> equals_Type_c(et, et2)
             | _ -> false 
             end ;             
}


function equals_Type_c
Boolean ::= t1::Type_c t2::Type_c
{ return t1.equals ;
  t1.equals_in = t2 ;
}
attribute equals occurs on Type_c ;
attribute equals_in<Type_c> occurs on Type_c ;

-- Var_t
terminal Var_t /[a-zA-Z][a-zA-Z0-9]*/ submits to {Keywords} ;

lexer class Keywords ;

terminal Int_t 'int' lexer classes {Keywords} ;
terminal Float_t 'float' lexer classes {Keywords} ;
terminal Array_t 'array' lexer classes {Keywords} ;
terminal Comma_t ',' ;
terminal Semi_t ';' ;

ignore terminal WhiteSpace_t /[\t\n\ ]+/ ;
