grammar Gram ;

attribute pp occurs on Root, Decls, Decl, Type ;
attribute show occurs on Root, Decls, Decl, Type ;

synthesized attribute put<a> :: a ;
attribute put<Root_c> occurs on Root ;
attribute put<Decls_c> occurs on Decls ;
attribute put<Decl_c> occurs on Decl ;
attribute put<Type_c> occurs on Type ;

synthesized attribute equals :: Boolean ;
inherited attribute equals_in<a> :: a ;

-- Root
nonterminal Root ;
abstract production root
r::Root ::= ds::Decls
{ r.pp = ds.pp ;
  r.show = "root( " ++ ds.show ++ " )";

  -- Simple inversion of the rewrite rule.
  r.put = case r of 
            root(ds) -> root_c(ds.put) 
          end ;

  -- The above could be optimized to
  -- r.put = root_c(ds.put)
  -- since the single pattern always matches

  -- The above could be expanded (uselessly) to the following: 
  {-
  r.put = case r of
            root(ds) -> 
            case ds of
              ds2 -> root_c(ds.put)
            end
          end ;
  -}
  {- This is the more general 'recursive' inversion.  It is the
     general case of applying the ideas that are needed to do the
     inversion of the rules for multiDecl_c constructs. -}

  r.equals = case r.equals_in of
             | root (ds2) -> equals_Decls(ds,ds2)
             | _ -> false 
             end ;             
}
function equals_Root
Boolean ::= t1::Root t2::Root
{ return t1.equals ;
  t1.equals_in = t2 ;
}
attribute equals occurs on Root ;
attribute equals_in<Root> occurs on Root ;

-- Decls
nonterminal Decls ;
abstract production consDecl
ds::Decls ::= d::Decl rest::Decls
{ ds.pp = d.pp ++ "\n" ++ rest.pp ;
  ds.show = "consDecl( " ++ d.show ++ ", " ++ rest.show ++ " )";
  ds.put = consDecl_c( d.put, rest.put ) ;

  ds.equals = case ds.equals_in of
             | consDecl (d2, rest2) -> equals_Decl(d,d2) && equals_Decls(rest,rest2)
             | _ -> false 
             end ;             
}

abstract production nilDecl
ds::Decls ::=
{ ds.pp = "" ;
  ds.show = "nilDecl()";
  ds.put = nilDecl_c() ;

  ds.equals = case ds.equals_in of
              | nilDecl() -> true
              | _ -> false
              end ;
}

function equals_Decls
Boolean ::= t1::Decls t2::Decls
{ return t1.equals ;
  t1.equals_in = t2 ;
}
attribute equals occurs on Decls ;
attribute equals_in<Decls> occurs on Decls ;


-- Decl
nonterminal Decl ;

abstract production nestedDecls
d::Decl ::= ds::Decls
{ d.pp = ds.pp ;
  d.show = "nestedDecls( " ++ ds.show ++ " )";
  d.put = 
    {- The inverted rewrite rule:
       nestedDecls(
         consDecl( varDecl( get(t), lexeme), 
                   consDecl(  ( get (multiDecl_c( t, rest, semi_t()) ) ),  nilDecl() ) )  )
       -> get( multiDecl_c( t, consVar_c( var_t(lexeme), comma_t(), rest), semi_t() ) )
     -}
    case d of
    | nestedDecls(
        consDecl( varDecl( t_a {-get(t)-}, lexeme), 
                  consDecl(  decl_a {-( get (multiDecl_c( t, rest, semi_t()) ) )-},  nilDecl() ) )  )
      -> case decl_a.put of
         | multiDecl_c ( t2, rest, semi )
           -> ( if equals_Type_c( t2, t_a.put )
                then multiDecl_c( t_a.put, consVar_c( terminal(Var_t, lexeme), terminal(Comma_t,","), rest), semi )
                else put_case_0 )
         | _ -> put_case_0
         end 
    | _ -> put_case_0
    end ;

  local attribute put_case_0 ::Decl_c = error("No matches on put on nestedDecl\n") ;

  {- Note that the inversion of rules here is not complete.

     I've not yet thought about how the system can fill these in,
     maybe using the "can-be" relationship.  Work to be done....
   -}

  d.equals = case d.equals_in of
             | nestedDecls (ds2) -> equals_Decls(ds,ds2)
             | _ -> false 
             end ;             
}

abstract production varDecl
d::Decl ::= t::Type v::String
{ d.pp = t.pp ++ " " ++ v ++ " ;" ;
  d.show = "varDecl( " ++ t.show ++ ", " ++ v ++ " )";
  d.put = 
    --  put( varDecl( get(t), lexeme ) )  ->  multiDecl_c( t, oneVar_c( var_t(lexeme) ), semi_t() )
    case d of
      varDecl(t, v) -> multiDecl_c( t.put,
                                    oneVar_c( terminal(Var_t, v) ), 
                                    terminal(Semi_t,";") ) 
    end ;

  d.equals = case d.equals_in of
             | varDecl( t2, v2) -> equals_Type(t, t2) && v == v2 
             | _ -> false 
             end ;             
}

function equals_Decl
Boolean ::= t1::Decl t2::Decl
{ return t1.equals ;
  t1.equals_in = t2 ;
}
attribute equals occurs on Decl ;
attribute equals_in<Decl> occurs on Decl ;


-- Type
nonterminal Type ;

abstract production intType
t::Type ::= 
{ t.pp = "int" ;
  t.show = "intType()";
  t.put = intType_c( terminal(Int_t, "int") ) ;

  t.equals = case t.equals_in of
             | intType() -> true
             | _ -> false 
             end ;             
}

abstract production floatType
t::Type ::= 
{ t.pp = "float" ;
  t.show = "floatType()";
  t.put = floatType_c( terminal(Float_t, "float") ) ;

  t.equals = case t.equals_in of
             | intType() -> true
             | _ -> false 
             end ;             
}

abstract production arrayType
t::Type ::= et::Type
{ t.pp = "array " ++ et.pp ;
  t.show = "arrayType()";
  t.put = arrayType_c( terminal(Array_t, "array"), et.put ) ;

  t.equals = case t.equals_in of
             | arrayType(et2) -> equals_Type(et, et2)
             | _ -> false 
             end ;             
}

function equals_Type
Boolean ::= t1::Type t2::Type
{ return t1.equals ;
  t1.equals_in = t2 ;
}
attribute equals occurs on Type ;
attribute equals_in<Type> occurs on Type ;


