module MainSL where

import Bidi
import Data.List
import Data.Maybe
import Data.Data
import Source2Silver
import View2Silver
import GetTransf2Silver
import PutTransf2Silver
import Terminals2Silver
import Transf2Silver

r :: IO()
r = do putStrLn "Generating Silver Files..."
       writeFile "Gram/Concrete_Generated.sv" (source2Silver sl_grammar_S)
       writeFile "Gram/Abstract_Generated.sv" (view2Silver   (sl_grammar_V, sl_grammar_S))
       writeFile "Gram/Get_Generated.sv"      (toSilverGet   getSL)
       writeFile "Gram/Put_Generated.sv"      (toSilverPut   (invert getSL, getSL))
       writeFile "Gram/Terminals.sv"          (initialize)
       putStrLn ("Silver Files generated in './Gram'.")

-- -- -- The grammars SL_S and SL_V are exactly the same except for the name of their productions, terminals and nonterminal. -- -- --
-- -- -- SimpleLanguage Grammar Source
data SL_Terminals_S = LeftSquareBrackets_S | RightSquareBrackets_S | Equal_S | Semicollon_S | Plus_S | Minus_S | Times_S | LeftParen_S | RightParen_S | INT_S | STR_S | IF_S | THEN_S | ELSE_S | PRINT_S | WHILE_S | REPEAT_S | UNTIL_S | VerticalBar_S | LeftBracket_S | RightBracket_S | ATTRIB_S
                  deriving (Show, Eq, Ord)

instance RegExp SL_Terminals_S where
    regExp LeftSquareBrackets_S  = "["
    regExp RightSquareBrackets_S = "]"
    regExp Equal_S               = "="
    regExp Semicollon_S          = ";"
    regExp Plus_S                = "+"
    regExp Minus_S               = "-"
    regExp Times_S               = "x"
    regExp LeftParen_S           = "("
    regExp RightParen_S          = ")"
    regExp INT_S                 = "INT"
    regExp STR_S                 = "STR"
    regExp IF_S                  = "IF"
    regExp THEN_S                = "THEN"
    regExp ELSE_S                = "ELSE"
    regExp PRINT_S               = "PRINT"
    regExp WHILE_S               = "WHILE"
    regExp REPEAT_S              = "REPEAT"
    regExp UNTIL_S               = "UNTIL"
    regExp VerticalBar_S         = "|"
    regExp LeftBracket_S         = "{"
    regExp RightBracket_S        = "}"
    regExp ATTRIB_S              = "ATTRIB"

data SL_Nonterminals_S = SL_S | DeclsList_S | Type_S | Stat_S | Expr_S | ListStat_S | Txpr_S | Fxpr_S
                     deriving (Show, Eq, Ord)

data SL_ProdNames_S = Begin_S | OneDecl_S | TipoInt_S | TipoString_S | VariousDecl_S | OneStat_S | VariousStat_S | Attrib_S | Sum_S | Sub_S | Et_S | Mul_S | Tf_S | Neg_S | Nest_S | Const_S | Variable_S | IfElse_S | Print_S | While_S | RepeatUntil_S
                  deriving (Show, Eq, Ord)
		
sl_grammar_S :: CFG SL_Terminals_S SL_Nonterminals_S SL_ProdNames_S
sl_grammar_S =  CFG
-- NonTerms and Prims
  [SL_S, DeclsList_S, Type_S, Stat_S, ListStat_S, Expr_S, Txpr_S, Fxpr_S] [String_Lit, Int_Lit]
-- Terms
  [LeftSquareBrackets_S, RightSquareBrackets_S, Equal_S, Semicollon_S, Plus_S, Minus_S, Times_S, LeftParen_S, RightParen_S, INT_S, STR_S, IF_S, THEN_S, ELSE_S, PRINT_S, WHILE_S, REPEAT_S, UNTIL_S, VerticalBar_S, LeftBracket_S, RightBracket_S, ATTRIB_S]
-- Prod Names
  [Begin_S, OneDecl_S, TipoInt_S, TipoString_S, VariousDecl_S, OneStat_S, VariousStat_S, Attrib_S, Const_S, Variable_S, Sum_S, Sub_S, Et_S, Mul_S, Tf_S, Neg_S, Nest_S, Const_S, Variable_S, IfElse_S, Print_S, While_S, RepeatUntil_S]
-- Prods
  [Prod Begin_S       SL_S        [Term LeftSquareBrackets_S, NonTerm DeclsList_S, Term VerticalBar_S, NonTerm ListStat_S, Term RightSquareBrackets_S],
   Prod OneDecl_S     DeclsList_S [NonTerm Type_S, Prim String_Lit, Term Semicollon_S],
   Prod VariousDecl_S DeclsList_S [NonTerm Type_S, Prim String_Lit, Term Semicollon_S, NonTerm DeclsList_S],
   Prod TipoInt_S     Type_S      [Term INT_S],
   Prod TipoString_S  Type_S      [Term STR_S],
   Prod OneStat_S     ListStat_S  [NonTerm Stat_S],
   Prod VariousStat_S ListStat_S  [NonTerm Stat_S, NonTerm ListStat_S],
   Prod Attrib_S      Stat_S      [Term ATTRIB_S, Prim String_Lit, Term Equal_S, NonTerm Expr_S, Term Semicollon_S],
   --   Prod IfElse      Stat      [Term IF, NonTerm Expr, Term THEN, Term LeftBracket, NonTerm ListStat, Term RightBracket, Term ELSE, Term LeftBracket, NonTerm ListStat, Term RightBracket],
   Prod While_S       Stat_S      [Term WHILE_S, NonTerm Expr_S, Term LeftBracket_S, NonTerm ListStat_S, Term RightBracket_S],
   Prod Print_S       Stat_S      [Term PRINT_S, NonTerm Expr_S, Term Semicollon_S],
   Prod RepeatUntil_S Stat_S      [Term REPEAT_S, Term LeftBracket_S, NonTerm ListStat_S, Term RightBracket_S, Term UNTIL_S, NonTerm Expr_S],

   Prod Sum_S         Expr_S      [NonTerm Expr_S, Term Plus_S,  NonTerm Txpr_S],
   Prod Sub_S         Expr_S      [NonTerm Expr_S, Term Minus_S, NonTerm Txpr_S],
   Prod Et_S          Expr_S      [NonTerm Txpr_S],
   Prod Mul_S         Txpr_S      [NonTerm Txpr_S, Term Times_S, NonTerm Fxpr_S],
   Prod Tf_S          Txpr_S      [NonTerm Fxpr_S],
   Prod Neg_S         Fxpr_S      [Term Minus_S, NonTerm Fxpr_S],
   Prod Nest_S        Fxpr_S      [Term LeftParen_S, NonTerm Expr_S, Term RightParen_S],
   Prod Const_S       Fxpr_S      [Prim Int_Lit],
   Prod Variable_S    Fxpr_S      [Prim String_Lit]
  ]

-- -- -- SimpleLanguage Grammar View
data SL_Terminals_V = LeftSquareBrackets_V | RightSquareBrackets_V | Equal_V | Semicollon_V | Plus_V | Minus_V | Times_V | LeftParen_V | RightParen_V | INT_V | STR_V | IF_V | THEN_V | ELSE_V | PRINT_V | WHILE_V | REPEAT_V | UNTIL_V | VerticalBar_V | LeftBracket_V | RightBracket_V | ATTRIB_V
                  deriving (Show, Eq, Ord)

instance RegExp SL_Terminals_V where
    regExp LeftSquareBrackets_V  = "["
    regExp RightSquareBrackets_V = "]"
    regExp Equal_V               = "="
    regExp Semicollon_V          = ";"
    regExp Plus_V                = "+"
    regExp Minus_V               = "-"
    regExp Times_V               = "x"
    regExp LeftParen_V           = "("
    regExp RightParen_V          = ")"
    regExp INT_V                 = "INT"
    regExp STR_V                 = "STR"
    regExp IF_V                  = "IF"
    regExp THEN_V                = "THEN"
    regExp ELSE_V                = "ELSE"
    regExp PRINT_V               = "PRINT"
    regExp WHILE_V               = "WHILE"
    regExp REPEAT_V              = "REPEAT"
    regExp UNTIL_V               = "UNTIL"
    regExp VerticalBar_V         = "|"
    regExp LeftBracket_V         = "{"
    regExp RightBracket_V        = "}"
    regExp ATTRIB_V              = "ATTRIB"

data SL_Nonterminals_V = SL_V | DeclsList_V | Type_V | Stat_V | Expr_V | ListStat_V | Txpr_V | Fxpr_V
                     deriving (Show, Eq, Ord)

data SL_ProdNames_V = Begin_V | OneDecl_V | TipoInt_V | TipoString_V | VariousDecl_V | OneStat_V | VariousStat_V | Attrib_V | Sum_V | Sub_V | Et_V | Mul_V | Tf_V | Neg_V | Nest_V | Const_V | Variable_V | IfElse_V | Print_V | While_V | RepeatUntil_V
                  deriving (Show, Eq, Ord)
    
sl_grammar_V :: CFG SL_Terminals_V SL_Nonterminals_V SL_ProdNames_V
sl_grammar_V =  CFG
-- NonTerms and Prims
  [SL_V, DeclsList_V, Type_V, Stat_V, ListStat_V, Expr_V, Txpr_V, Fxpr_V] [String_Lit, Int_Lit]
-- Terms
  [LeftSquareBrackets_V, RightSquareBrackets_V, Equal_V, Semicollon_V, Plus_V, Minus_V, Times_V, LeftParen_V, RightParen_V, INT_V, STR_V, IF_V, THEN_V, ELSE_V, PRINT_V, WHILE_V, REPEAT_V, UNTIL_V, VerticalBar_V, LeftBracket_V, RightBracket_V, ATTRIB_V]
-- Prod Names
  [Begin_V, OneDecl_V, TipoInt_V, TipoString_V, VariousDecl_V, OneStat_V, VariousStat_V, Attrib_V, Const_V, Variable_V, Sum_V, Sub_V, Et_V, Mul_V, Tf_V, Neg_V, Nest_V, Const_V, Variable_V, IfElse_V, Print_V, While_V, RepeatUntil_V]
-- Prods
  [Prod Begin_V       SL_V        [Term LeftSquareBrackets_V, NonTerm DeclsList_V, Term VerticalBar_V, NonTerm ListStat_V, Term RightSquareBrackets_V],
   Prod OneDecl_V     DeclsList_V [NonTerm Type_V, Prim String_Lit, Term Semicollon_V],
   Prod VariousDecl_V DeclsList_V [NonTerm Type_V, Prim String_Lit, Term Semicollon_V, NonTerm DeclsList_V],
   Prod TipoInt_V     Type_V      [Term INT_V],
   Prod TipoString_V  Type_V      [Term STR_V],
   Prod OneStat_V     ListStat_V  [NonTerm Stat_V],
   Prod VariousStat_V ListStat_V  [NonTerm Stat_V, NonTerm ListStat_V],
   Prod Attrib_V      Stat_V      [Term ATTRIB_V, Prim String_Lit, Term Equal_V, NonTerm Expr_V, Term Semicollon_V],
   --   Prod IfElse      Stat      [Term IF, NonTerm Expr, Term THEN, Term LeftBracket, NonTerm ListStat, Term RightBracket, Term ELSE, Term LeftBracket, NonTerm ListStat, Term RightBracket],
   Prod While_V       Stat_V      [Term WHILE_V, NonTerm Expr_V, Term LeftBracket_V, NonTerm ListStat_V, Term RightBracket_V],
   Prod Print_V       Stat_V      [Term PRINT_V, NonTerm Expr_V, Term Semicollon_V],
   Prod RepeatUntil_V Stat_V      [Term REPEAT_V, Term LeftBracket_V, NonTerm ListStat_V, Term RightBracket_V, Term UNTIL_V, NonTerm Expr_V],

   Prod Sum_V         Expr_V      [NonTerm Expr_V, Term Plus_V,  NonTerm Txpr_V],
   Prod Sub_V         Expr_V      [NonTerm Expr_V, Term Minus_V, NonTerm Txpr_V],
   Prod Et_V          Expr_V      [NonTerm Txpr_V],
   Prod Mul_V         Txpr_V      [NonTerm Txpr_V, Term Times_V, NonTerm Fxpr_V],
   Prod Tf_V          Txpr_V      [NonTerm Fxpr_V],
   Prod Neg_V         Fxpr_V      [Term Minus_V, NonTerm Fxpr_V],
   Prod Nest_V        Fxpr_V      [Term LeftParen_V, NonTerm Expr_V, Term RightParen_V],
   Prod Const_V       Fxpr_V      [Prim Int_Lit],
   Prod Variable_V    Fxpr_V      [Prim String_Lit]
  ]


    
{-
-- -- -- Transformation from SL_S to SL_V
ntSL2 :: NTFunc SL_Nonterminals SL_Nonterminals
ntSL2 = [(SL,[SL]),(DeclsList,[DeclsList]),(Type,[Type]),(ListStat,[ListStat]),(Stat,[Stat]),(Expr,[Expr]),(Txpr,[Txpr]),(Fxpr,[Fxpr])]

rulesSL2 :: [Rule SL_Terminals SL_Nonterminals SL_ProdNames 
                 SL_Terminals SL_Nonterminals SL_ProdNames]
rulesSL2 =
 [  ( Pattern (Begin,SL)              [P_Term LeftSquareBrackets, P_Arg('a',DeclsList), P_Term VerticalBar, P_Arg('b',ListStat), P_Term RightSquareBrackets],
      Pattern (Begin,SL)              [P_Term LeftSquareBrackets, P_Arg('a',DeclsList), P_Term VerticalBar, P_Arg('b',ListStat), P_Term RightSquareBrackets] )

  , ( Pattern (OneDecl,DeclsList)     [P_Arg('a',Type), P_Prim ('i',String_Lit), P_Term Semicollon],
      Pattern (OneDecl,DeclsList)     [P_Arg('a',Type), P_Prim ('i',String_Lit), P_Term Semicollon])
  , ( Pattern (VariousDecl,DeclsList) [P_Arg('a',Type), P_Prim ('i',String_Lit), P_Term Semicollon, P_Arg('b',DeclsList)],
      Pattern (VariousDecl,DeclsList) [P_Arg('a',Type), P_Prim ('i',String_Lit), P_Term Semicollon, P_Arg('b',DeclsList)])
 
  , ( Pattern (TipoInt,Type)          [P_Term INT],  Pattern (TipoInt,Type)          [P_Term INT])
  , ( Pattern (TipoString,Type)       [P_Term STR],  Pattern (TipoString,Type)       [P_Term STR])

  , ( Pattern (OneStat,ListStat)      [P_Arg('a',Stat)],                     Pattern (OneStat, ListStat)     [P_Arg('a',Stat)])
  , ( Pattern (VariousStat,ListStat)  [P_Arg('a',Stat),P_Arg('b',ListStat)], Pattern (VariousStat, ListStat) [P_Arg('a',Stat),P_Arg('b',ListStat)])

  , ( Pattern (Attrib,Stat)           [P_Term ATTRIB, P_Prim ('i',String_Lit), P_Term Equal, P_Arg('a',Expr), P_Term Semicollon],
      Pattern (Attrib,Stat)           [P_Term ATTRIB, P_Prim ('i',String_Lit), P_Term Equal, P_Arg('a',Expr), P_Term Semicollon])
--  , ( Pattern (IfElse,Stat)           [P_Term IF, P_Arg('a',Expr), P_Term THEN, P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket, P_Term ELSE, P_Term LeftBracket, P_Arg('c',ListStat), P_Term RightBracket],
--      Pattern (IfElse,Stat)           [P_Term IF, P_Arg('a',Expr), P_Term THEN, P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket, P_Term ELSE, P_Term LeftBracket, P_Arg('c',ListStat), P_Term RightBracket])
  , ( Pattern (While,Stat)            [P_Term WHILE, P_Arg('a',Expr), P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket],
      Pattern (While,Stat)            [P_Term WHILE, P_Arg('a',Expr), P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket])
  , ( Pattern (Print,Stat)            [P_Term PRINT, P_Arg('a',Expr), P_Term Semicollon],
      Pattern (Print,Stat)            [P_Term PRINT, P_Arg('a',Expr), P_Term Semicollon])
  , ( Pattern (RepeatUntil,Stat)      [P_Term REPEAT, P_Term LeftBracket, P_Arg('a',ListStat), P_Term RightBracket, P_Term UNTIL, P_Arg('b',Expr)],
      Pattern (RepeatUntil,Stat)      [P_Term REPEAT, P_Term LeftBracket, P_Arg('a',ListStat), P_Term RightBracket, P_Term UNTIL, P_Arg('b',Expr)] )
  , ( Pattern (RepeatUntil,Stat)      [P_Term REPEAT, P_Term LeftBracket, P_Arg('a',ListStat), P_Term RightBracket, P_Term UNTIL, P_Arg('b',Expr)],
      Pattern (While,Stat)            [P_Term WHILE, P_Arg('b',Expr), P_Term LeftBracket, P_Arg('a',ListStat), P_Term RightBracket ])

  , ( Pattern (Sub,Expr)              [Pattern (Et,Expr)   [Pattern (Tf,Txpr)  [Pattern (Const,Fxpr) [P_Lexemme ('i',"0",Int_Lit)]]], P_Term Minus, P_Arg ('a',Txpr)],
      Pattern (Et,Expr)               [Pattern (Tf, Txpr)  [Pattern (Neg,Fxpr) [P_Term Minus, Pattern (Nest, Expr) [P_Term LeftParen, Pattern (Et,Expr) [P_Arg ('a',Txpr)], P_Term RightParen]]]])

  , ( Pattern (Sum,Expr)              [P_Arg ('a',Expr), P_Term Plus,  P_Arg ('b',Txpr)], Pattern (Sum,Expr)              [P_Arg ('a',Expr), P_Term Plus,  P_Arg ('b',Txpr)])
  , ( Pattern (Sub,Expr)              [P_Arg ('a',Expr), P_Term Minus, P_Arg ('b',Txpr)], Pattern (Sub,Expr)              [P_Arg ('a',Expr), P_Term Minus, P_Arg ('b',Txpr)])
  , ( Pattern (Et,Expr)               [P_Arg ('a',Txpr)],                                 Pattern (Et,Expr)               [P_Arg ('a',Txpr)] )
  , ( Pattern (Mul,Txpr)              [P_Arg ('a',Txpr), P_Term Times, P_Arg ('b',Fxpr)], Pattern (Mul,Txpr)              [P_Arg ('a',Txpr), P_Term Times, P_Arg ('b',Fxpr)])
  , ( Pattern (Tf,Txpr)               [P_Arg ('a',Fxpr)]                                , Pattern (Tf,Txpr)               [P_Arg ('a',Fxpr)])
  , ( Pattern (Neg,Fxpr)              [P_Term Minus, P_Arg ('a',Fxpr)]                  , Pattern (Neg,Fxpr)              [P_Term Minus, P_Arg ('a',Fxpr)])
  , ( Pattern (Nest,Fxpr)             [P_Term LeftParen, P_Arg ('a',Expr), P_Term RightParen], Pattern (Nest,Fxpr) [P_Term LeftParen, P_Arg ('a',Expr), P_Term RightParen])
  , ( Pattern (Const,Fxpr)            [P_Prim ('a',Int_Lit)],    Pattern (Const,Fxpr)            [P_Prim ('a',Int_Lit)])
  , ( Pattern (Variable,Fxpr)         [P_Prim ('a',String_Lit)], Pattern (Variable,Fxpr)         [P_Prim ('a',String_Lit)])
 ]

getSL2 :: Transf SL_Terminals SL_Nonterminals SL_ProdNames
                SL_Terminals SL_Nonterminals SL_ProdNames
getSL2 =  Transf ntSL2 rulesSL2

-}



-- -- -- Transformation from SL_S to SL_V
-- This transformation corresponds to the identity
ntSL :: NTFunc SL_Nonterminals_S SL_Nonterminals_V
ntSL = [(SL_S,[SL_V]),(DeclsList_S,[DeclsList_V]),(Type_S,[Type_V]),(ListStat_S,[ListStat_V]),(Stat_S,[Stat_V]),(Expr_S,[Expr_V]),(Txpr_S,[Txpr_V]),(Fxpr_S,[Fxpr_V])]

rulesSL :: [Rule SL_Terminals_S SL_Nonterminals_S SL_ProdNames_S 
                 SL_Terminals_V SL_Nonterminals_V SL_ProdNames_V]
rulesSL =
 [  ( Pattern (Begin_S,SL_S)              [P_Term LeftSquareBrackets_S, P_Arg('a',DeclsList_S), P_Term VerticalBar_S, P_Arg('b',ListStat_S), P_Term RightSquareBrackets_S],
      Pattern (Begin_V,SL_V)              [P_Term LeftSquareBrackets_V, P_Arg('a',DeclsList_V), P_Term VerticalBar_V, P_Arg('b',ListStat_V), P_Term RightSquareBrackets_V] )

  , ( Pattern (OneDecl_S,DeclsList_S)     [P_Arg('a',Type_S), P_Prim ('i',String_Lit), P_Term Semicollon_S],
      Pattern (OneDecl_V,DeclsList_V)     [P_Arg('a',Type_V), P_Prim ('i',String_Lit), P_Term Semicollon_V])
  , ( Pattern (VariousDecl_S,DeclsList_S) [P_Arg('a',Type_S), P_Prim ('i',String_Lit), P_Term Semicollon_S, P_Arg('b',DeclsList_S)],
      Pattern (VariousDecl_V,DeclsList_V) [P_Arg('a',Type_V), P_Prim ('i',String_Lit), P_Term Semicollon_V, P_Arg('b',DeclsList_V)])
 
  , ( Pattern (TipoInt_S,Type_S)          [P_Term INT_S],  Pattern (TipoInt_V,Type_V)          [P_Term INT_V])
  , ( Pattern (TipoString_S,Type_S)       [P_Term STR_S],  Pattern (TipoString_V,Type_V)       [P_Term STR_V])

  , ( Pattern (OneStat_S,ListStat_S)      [P_Arg('a',Stat_S)],                     Pattern (OneStat_V, ListStat_V)     [P_Arg('a',Stat_V)])
  , ( Pattern (VariousStat_S,ListStat_S)  [P_Arg('a',Stat_S),P_Arg('b',ListStat_S)], Pattern (VariousStat_V, ListStat_V) [P_Arg('a',Stat_V),P_Arg('b',ListStat_V)])

  , ( Pattern (Attrib_S,Stat_S)           [P_Term ATTRIB_S, P_Prim ('i',String_Lit), P_Term Equal_S, P_Arg('a',Expr_S), P_Term Semicollon_S],
      Pattern (Attrib_V,Stat_V)           [P_Term ATTRIB_V, P_Prim ('i',String_Lit), P_Term Equal_V, P_Arg('a',Expr_V), P_Term Semicollon_V])
--  , ( Pattern (IfElse,Stat)           [P_Term IF, P_Arg('a',Expr), P_Term THEN, P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket, P_Term ELSE, P_Term LeftBracket, P_Arg('c',ListStat), P_Term RightBracket],
--      Pattern (IfElse,Stat)           [P_Term IF, P_Arg('a',Expr), P_Term THEN, P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket, P_Term ELSE, P_Term LeftBracket, P_Arg('c',ListStat), P_Term RightBracket])
  , ( Pattern (While_S,Stat_S)            [P_Term WHILE_S, P_Arg('a',Expr_S), P_Term LeftBracket_S, P_Arg('b',ListStat_S), P_Term RightBracket_S],
      Pattern (While_V,Stat_V)            [P_Term WHILE_V, P_Arg('a',Expr_V), P_Term LeftBracket_V, P_Arg('b',ListStat_V), P_Term RightBracket_V])
  , ( Pattern (Print_S,Stat_S)            [P_Term PRINT_S, P_Arg('a',Expr_S), P_Term Semicollon_S],
      Pattern (Print_V,Stat_V)            [P_Term PRINT_V, P_Arg('a',Expr_V), P_Term Semicollon_V])
  , ( Pattern (RepeatUntil_S,Stat_S)      [P_Term REPEAT_S, P_Term LeftBracket_S, P_Arg('a',ListStat_S), P_Term RightBracket_S, P_Term UNTIL_S, P_Arg('b',Expr_S)],
      Pattern (RepeatUntil_V,Stat_V)      [P_Term REPEAT_V, P_Term LeftBracket_V, P_Arg('a',ListStat_V), P_Term RightBracket_V, P_Term UNTIL_V, P_Arg('b',Expr_V)])

  , ( Pattern (Sum_S,Expr_S)              [P_Arg ('a',Expr_S), P_Term Plus_S,  P_Arg ('b',Txpr_S)]      , Pattern (Sum_V,Expr_V)      [P_Arg ('a',Expr_V), P_Term Plus_V,  P_Arg ('b',Txpr_V)])
  , ( Pattern (Sub_S,Expr_S)              [P_Arg ('a',Expr_S), P_Term Minus_S, P_Arg ('b',Txpr_S)]      , Pattern (Sub_V,Expr_V)      [P_Arg ('a',Expr_V), P_Term Minus_V, P_Arg ('b',Txpr_V)])
  , ( Pattern (Et_S,Expr_S)               [P_Arg ('a',Txpr_S)]                                          , Pattern (Et_V,Expr_V)       [P_Arg ('a',Txpr_V)] )
  , ( Pattern (Mul_S,Txpr_S)              [P_Arg ('a',Txpr_S), P_Term Times_S, P_Arg ('b',Fxpr_S)]      , Pattern (Mul_V,Txpr_V)      [P_Arg ('a',Txpr_V), P_Term Times_V, P_Arg ('b',Fxpr_V)])
  , ( Pattern (Tf_S,Txpr_S)               [P_Arg ('a',Fxpr_S)]                                          , Pattern (Tf_V,Txpr_V)       [P_Arg ('a',Fxpr_V)])
  , ( Pattern (Neg_S,Fxpr_S)              [P_Term Minus_S, P_Arg ('a',Fxpr_S)]                          , Pattern (Neg_V,Fxpr_V)      [P_Term Minus_V, P_Arg ('a',Fxpr_V)])
  , ( Pattern (Nest_S,Fxpr_S)             [P_Term LeftParen_S, P_Arg ('a',Expr_S), P_Term RightParen_S] , Pattern (Nest_V,Fxpr_V)     [P_Term LeftParen_V, P_Arg ('a',Expr_V), P_Term RightParen_V])
  , ( Pattern (Const_S,Fxpr_S)            [P_Prim ('a',Int_Lit)]                                        , Pattern (Const_V,Fxpr_V)    [P_Prim ('a',Int_Lit)])
  , ( Pattern (Variable_S,Fxpr_S)         [P_Prim ('a',String_Lit)]                                     , Pattern (Variable_V,Fxpr_V) [P_Prim ('a',String_Lit)])
 ]

getSL :: Transf SL_Terminals_S SL_Nonterminals_S SL_ProdNames_S
                SL_Terminals_V SL_Nonterminals_V SL_ProdNames_V
getSL =  Transf ntSL rulesSL
