module Source2Silver where

import Bidi
import Data.List

instance (Show term, Show nonTerm, Show prod_name, RegExp term) => Source2Silver (CFG term nonTerm prod_name) where
  source2Silver gram@(CFG nTerms prims terms prodName prods) =
      let header = "grammar Gram;\n" -- by default all grammars are called Gram
          -- Print the declaration of attributes
		  -- REPLACED - Done in Terminals2Silver
          s1     = ""--"synthesized attribute show :: String;\nsynthesized attribute pp :: String;\n" 
          -- Print the declaration of NonTerminals
          s2     = unlines $ map auxNT (cfgNonTerms gram)
          -- Print the declaration of Primitive types
		  -- REPLACED - Done in Terminals2Silver		  
          s3     = "" --unlines . (map (\x -> "terminal "   ++show x++" "++regExp x  ++";"))  $ prims
          -- Print the declaration of Terminals
          s4     = unlines . (map (\x -> "terminal "   ++show x++" '"++regExp x ++"';")) $ terms
          -- Print all the grammar productions
          s5     = unlines . map source2Silver $ prods
      in  unlines [header,s1,s2,s3,s4,s5]
      where
          auxNT nt = "nonterminal "++show nt++" with pp, show;"

-- -- Attribute initialization    
instance (Show prod_name, Show nonTerm, Show term, RegExp term) => Source2Silver (Prod prod_name nonTerm term) where
  source2Silver prod@(Prod prod_name lhsNonTerm rhs) =
      let s1        = "concrete production p_" ++ show prod_name
          s2        = "this::" ++ show lhsNonTerm ++ " ::= " ++ (intercalate " " $ map auxRHS (zip ['a'..'z'] rhs))
          attr_show = attributeShow prod
          attr_pp   = attributePP   prod		  
      in  unlines [s1,s2,"{",attr_show++";",attr_pp++";","}"]
      where
          auxRHS (c, NonTerm x) = [c]++"::"++show x
          auxRHS (c, Term    x) = [c]++"::"++show x
          auxRHS (c, Prim    x) = [c]++"::"++show x

-- -- Attribute Show
attributeShow :: (RegExp term, Show prod_name) => (Prod prod_name nonTerm term) -> String
attributeShow prod@(Prod prod_name lhsNonTerm rhs) =
      let s1 = intercalate " ++ \",\" ++ " $ map aux (zip ['a'..'z'] rhs)
      in  "  this.show = \""++show prod_name++"(\" ++ "++ s1 ++" ++ \")\""
      where
          aux (c, NonTerm x) = [c]++".show"
          aux (c, Term    x) = "\" "++regExp x++" \""
          aux (c, Prim    x) = [c]++".lexeme"

-- -- Attribute PP
attributePP :: (RegExp term, Show prod_name) => (Prod prod_name nonTerm term) -> String
attributePP prod@(Prod prod_name lhsNonTerm rhs) =
      let s1 = intercalate " ++ \" \" ++ " $ map aux (zip ['a'..'z'] rhs)
      in  "  this.pp = " ++ s1
      where
          aux (c, NonTerm x) = [c]++".pp"
          aux (c, Term    x) = "\"" ++ regExp x ++ "\""
          aux (c, Prim    x) = [c]++".lexeme"
		  
