{-# LANGUAGE FlexibleInstances #-}

module View2Silver where

import Bidi
import Data.List

instance (Show termV, Show nonTermV, Show prod_nameV, RegExp termV, Show nonTermS) =>
  View2Silver (CFG termV nonTermV prod_nameV,CFG termS nonTermS prod_nameS) where
  view2Silver (gram1@(CFG nTermsV primsV termsV prodNameV prodsV),gram2@(CFG nTermsS primsS termsS prodNameS prodsS)) =
      let header = "grammar Gram;\n" -- by default all grammars are called Gram
          -- Print the declaration of attributes
		  -- REPLACED - Done in Terminals2Silver
          s1     = ""--"synthesized attribute show :: String;\nsynthesized attribute pp :: String;\n" 
          -- Print the declaration of NonTerminals
          s2     = unlines $ map auxNT (cfgNonTerms gram1)
          -- Print the declaration of Primitive types
		  -- REPLACED - Done in Terminals2Silver		  
          s3     = "" --unlines . (map (\x -> "terminal "   ++show x++" "++regExp x  ++";"))  $ prims
          -- Print the declaration of Terminals
          s4     = unlines . (map (\x -> "terminal "   ++show x++" '"++regExp x ++"';")) $ termsV
          -- Generating the prods for the links back
          s5     = generate_links_back gram2
          -- Print all the grammar productions
          s6     = unlines . map view2Silver $ prodsV
      in  concat [header,s1,"\n",s2,"\n",s3,s4,s5,"\n\n",s6]
      where
          auxNT nt = "nonterminal "++show nt++" with pp, show, link;"

-- -- Attribute initialization    
instance (Show prod_name, Show nonTerm, Show term, RegExp term) => View2Silver (Prod prod_name nonTerm term) where
  view2Silver prod@(Prod prod_name lhsNonTerm rhs) =
      let s1        = "abstract production p_" ++ show prod_name
          s2        = "this::" ++ show lhsNonTerm ++ " ::= " ++ (intercalate " " $ map auxRHS (zip ['a'..'z'] rhs))
          attr_show = attributeShow prod
          attr_pp   = attributePP   prod		  
      in  unlines [s1,s2,"{",attr_show++";",attr_pp++";","}"]
      where
          auxRHS (c, NonTerm x) = [c]++"::"++show x
          auxRHS (c, Term    x) = [c]++"::"++show x
          auxRHS (c, Prim    x) = [c]++"::"++show x

-- -- Attribute Show
attributeShow :: (RegExp term, Show prod_name) => (Prod prod_name nonTerm term) -> String
attributeShow prod@(Prod prod_name lhsNonTerm rhs) =
      let s1 = intercalate " ++ \",\" ++ " $ map aux (zip ['a'..'z'] rhs)
      in  "  this.show = \""++show prod_name++"(\" ++ "++ s1 ++" ++ \")\""
      where
          aux (c, NonTerm x) = [c]++".show"
          aux (c, Term    x) = "\" "++regExp x++" \""
          aux (c, Prim    x) = [c]++".lexeme"

-- -- Attribute PP
attributePP :: (RegExp term, Show prod_name) => (Prod prod_name nonTerm term) -> String
attributePP prod@(Prod prod_name lhsNonTerm rhs) =
      let s1 = intercalate " ++ \" \" ++ " $ map aux (zip ['a'..'z'] rhs)
      in  "  this.pp = " ++ s1
      where
          aux (c, NonTerm x) = [c]++".pp"
          aux (c, Term    x) = "\"" ++ regExp x ++ "\""
          aux (c, Prim    x) = [c]++".lexeme"

-- The 'S' stands for Source, since this is the source grammar. Am I smart or what?
generate_links_back :: (Show nonTermS) => (CFG termS nonTermS prod_nameS) -> String
generate_links_back gramS = let s1 = "-- Links Back\nannotation link :: Link;\nnonterminal Link;"
                                s2 = unlines $ map aux (cfgNonTerms gramS)
                                s3 = "abstract production none\nthis::Link ::=\n{}"
                            in  concat [s1,"\n\n",s2,s3,"\n------"]
                            where aux nt = let s1 = "abstract production link"++show nt
                                               s2 = "this::Link ::= e::"++show nt
                                               s3 = "{}"
                                           in  unlines [s1,s2,s3]






















		  
