grammar Gram;

synthesized attribute to_E :: E;
synthesized attribute to_T :: T;
synthesized attribute to_F :: F;

nonterminal ConcreteE with pp, show, to_E;
nonterminal ConcreteT with pp, show, to_T;
nonterminal ConcreteF with pp, show, to_F;

terminal LeftParen '(';
terminal RightParen ')';
terminal Plus '+';
terminal Minus '-';
terminal Ex '*';

concrete production p_ConcreteAdd
this::ConcreteE ::= a::ConcreteE Plus c::ConcreteT
{
  this.show = "Add(" ++ a.show ++ "," ++ "+" ++ "," ++ c.show ++ ")";
  this.pp = a.pp ++ " " ++ "+" ++ " " ++ c.pp;
  this.to_E = p_Add(a.to_E, c.to_T);
}

concrete production p_ConcreteSub
this::ConcreteE ::= a::ConcreteE Minus c::ConcreteT
{
  this.show = "Sub(" ++ a.show ++ "," ++ "-" ++ "," ++ c.show ++ ")";
  this.pp = a.pp ++ " " ++ "-" ++ " " ++ c.pp;
  this.to_E = p_Sub(a.to_E, c.to_T);
}

concrete production p_ConcreteEt
this::ConcreteE ::= a::ConcreteT
{
  this.show = "Et(" ++ a.show ++ ")";
  this.pp = a.pp;
  this.to_E = p_Et(a.to_T);
}

concrete production p_ConcreteMul
this::ConcreteT ::= a::ConcreteT Ex c::ConcreteF
{
  this.show = "Mul(" ++ a.show ++ "," ++ "*" ++ "," ++ c.show ++ ")";
  this.pp = a.pp ++ " " ++ "*" ++ " " ++ c.pp;
  this.to_T = p_Mul(a.to_T, c.to_F);
}

concrete production p_ConcreteTf
this::ConcreteT ::= a::ConcreteF
{
  this.show = "Tf(" ++ a.show ++ ")";
  this.pp = a.pp;
  this.to_T = p_Tf(a.to_F);
}

concrete production p_ConcreteNeg
this::ConcreteF ::= Minus b::ConcreteF
{
  this.show = "Neg(" ++ "-" ++ "," ++ b.show ++ ")";
  this.pp = "-" ++ " " ++ b.pp;
  this.to_F = p_Neg(b.to_F);
}

concrete production p_ConcreteNest
this::ConcreteF ::= LeftParen b::ConcreteE RightParen
{
  this.show = "Nest(" ++ "(" ++ "," ++ b.show ++ "," ++ ")" ++ ")";
  this.pp = "(" ++ " " ++ b.pp ++ " " ++ ")";
  this.to_F = p_Nest(b.to_E);
}

concrete production p_ConcreteValue
this::ConcreteF ::= a::Int_Lit
{
  this.show = "Value(" ++ a.lexeme ++ ")";
  this.pp = a.lexeme;
  this.to_F = p_Value(int_NT( a.lexeme ));
}

