module Source2Abstract where

import Data.List
import Bidi

class Source2Abstract a where
	source2Abstract :: a -> String

instance (Show term, Show nonTerm, Show prod_name, RegExp term) => Source2Abstract (CFG term nonTerm prod_name) where
  source2Abstract gram@(CFG nTerms prims terms prodName prods) =
      let header = "grammar Gram;" -- by default the grammar is called Gram
          -- Print the declaration of attributes
          s1     = unlines $ map auxAttribs (cfgNonTerms gram)
          -- Print the declaration of NonTerminals
          s2     = unlines $ map auxNT (cfgNonTerms gram)
          -- Print all the grammar productions
          s3     = unlines . map source2Abstract $ prods
      in  concat [header,"\n\n",s1,"\n",s2,"\n",s3]
      where
          auxNT      nt = "nonterminal "++show nt++" with show, to_Concrete"++show nt++";"
          auxAttribs nt = "synthesized attribute to_Concrete"++show nt++" :: Concrete"++show nt++";"

instance (Show prod_name, Show nonTerm, Show term, RegExp term) => Source2Abstract (Prod prod_name nonTerm term) where
  source2Abstract prod@(Prod prod_name lhsNonTerm rhs) =
      let s1               = "abstract production p_" ++ show prod_name
          s2               = "this::" ++ show lhsNonTerm ++ " ::= " ++ (intercalate " " $ map auxRHS (zip ['a'..'z'] rhs))
          attr_show        = attributeShow prod
          attr_to_concrete = attributeToConcrete prod
      in  unlines [s1,s2,"{",attr_show,attr_to_concrete,"}"]
      where
          auxRHS (c, NonTerm x) = [c]++"::"++show x
          auxRHS (c, Term    x) = ""
          auxRHS (c, Prim    x) = [c]++"::"++literal2Type x

-- -- Attribute Show
attributeShow :: (RegExp term, Show prod_name) => (Prod prod_name nonTerm term) -> String
attributeShow prod@(Prod prod_name lhsNonTerm rhs) =
      let s1 = intercalate " ++ \",\" ++ " . filter (/= "") $ map aux (zip ['a'..'z'] rhs)
      in  "  this.show = \""++show prod_name++"(\" ++ "++ s1 ++" ++ \")\";"
      where
          aux (c, NonTerm x) = [c]++".show"
          aux (c, Term    x) = ""
          aux (c, Prim    x) = [c]++".getInfo"

-- -- Attribute to transform back to the Concrete form
attributeToConcrete :: (RegExp term, Show prod_name, Show nonTerm, Show term) => (Prod prod_name nonTerm term) -> String
attributeToConcrete prod@(Prod prod_name lhsNonTerm rhs) =
      let s1 = intercalate ", " . filter (/= "") $ map aux (zip ['a'..'z'] rhs)
      in  "  this.to_Concrete"++show lhsNonTerm++" = "++"p_Concrete"++show prod_name++"( "++s1++" );"
      where
          aux (c, NonTerm x) = [c]++".to_Concrete"++show x
          aux (c, Term    x) = "terminal ("++show x++", \""++regExp x++"\")"
          aux (c, Prim    x) = "terminal ("++show x++", "++[c]++".getInfo"++")"
