module Source2Concrete where

import Data.List
import Bidi

class Source2Concrete a where
	source2Concrete :: a -> String

instance (Show term, Show nonTerm, Show prod_name, RegExp term) => Source2Concrete (CFG term nonTerm prod_name) where
  source2Concrete gram@(CFG nTerms prims terms prodName prods) =
      let header = "grammar Gram;" -- by default the grammar is called Gram
          -- Print the declaration of attributes
          s1     = unlines $ map auxAttribs (cfgNonTerms gram)
          -- Print the declaration of NonTerminals
          s2     = unlines $ map auxNT (cfgNonTerms gram)
                    -- Print the declaration of Terminals
          s3     = unlines . (map (\x -> "terminal "   ++show x++" '"++regExp x ++"';")) $ terms
          -- Print all the grammar productions
          s4     = unlines . map source2Concrete $ prods
      in  concat [header,"\n\n",s1,"\n",s2,"\n",s3,"\n",s4]
      where
          auxNT      nt = "nonterminal Concrete"++show nt++" with pp, show, to_"++show nt++";"
          auxAttribs nt = "synthesized attribute to_"++show nt++" :: "++show nt++";"

instance (Show prod_name, Show nonTerm, Show term, RegExp term) => Source2Concrete (Prod prod_name nonTerm term) where
  source2Concrete prod@(Prod prod_name lhsNonTerm rhs) =
      let s1        = "concrete production p_Concrete" ++ show prod_name
          s2        = "this::Concrete" ++ show lhsNonTerm ++ " ::= " ++ (intercalate " " $ map auxRHS (zip ['a'..'z'] rhs))
          attr_show = attributeShow prod
          attr_pp   = attributePP   prod
          attr_to_abstract = attributeToAbstract prod
	  in  unlines [s1,s2,"{",attr_show,attr_pp,attr_to_abstract,"}"]
      where
          auxRHS (c, NonTerm x) = [c]++"::Concrete"++show x
          auxRHS (c, Term    x) = show x
          auxRHS (c, Prim    x) = [c]++"::"++show x

-- -- Attribute Show
attributeShow :: (RegExp term, Show prod_name) => (Prod prod_name nonTerm term) -> String
attributeShow prod@(Prod prod_name lhsNonTerm rhs) =
      let s1 = intercalate " ++ \",\" ++ " . filter (/= "") $ map aux (zip ['a'..'z'] rhs)
      in  "  this.show = \""++show prod_name++"(\" ++ "++ s1 ++" ++ \")\";"
      where
          aux (c, NonTerm x) = [c]++".show"
          aux (c, Term    x) = "\""++regExp x++"\""
          aux (c, Prim    x) = [c]++".lexeme"

-- -- Attribute PP
attributePP :: (RegExp term, Show prod_name) => (Prod prod_name nonTerm term) -> String
attributePP prod@(Prod prod_name lhsNonTerm rhs) =
      let s1 = intercalate " ++ \" \" ++ " . filter (/= "") $ map aux (zip ['a'..'z'] rhs)
      in  "  this.pp = " ++ s1 ++ ";"
      where
          aux (c, NonTerm x) = [c]++".pp"
          aux (c, Term    x) = "\"" ++ regExp x ++ "\""
          aux (c, Prim    x) = [c]++".lexeme"

-- -- Attribute to transform back to the Abstract form
attributeToAbstract :: (RegExp term, Show prod_name, Show nonTerm, Show term) => (Prod prod_name nonTerm term) -> String
attributeToAbstract prod@(Prod prod_name lhsNonTerm rhs) =
      let s1 = intercalate ", " . filter (/= "") $ map aux (zip ['a'..'z'] rhs)
      in  "  this.to_"++show lhsNonTerm++" = "++"p_"++show prod_name++"("++s1++");"
      where
          aux (c, NonTerm x) = [c]++".to_"++show x
          aux (c, Term    x) = ""
          aux (c, Prim    x) = literal2Prod x++"( "++[c]++".lexeme"++" )"
