module View2Abstract where

import Data.List
import Bidi

class View2Abstract a where
	view2Abstract :: a -> String

instance (Show term, Show nonTerm, Show prod_name, RegExp term) => View2Abstract (CFG term nonTerm prod_name) where
  view2Abstract gram@(CFG nTerms prims terms prodName prods) =
      let header = "grammar Gram;" -- by default the grammar is called Gram
          -- Print the declaration of NonTerminals
          s1     = unlines $ map auxNT (cfgNonTerms gram)
          -- Print all the grammar productions
          s2     = unlines . map view2Abstract $ prods
      in  concat [header,"\n\n",s1,"\n",s2]
      where
          auxNT      nt = "nonterminal "++show nt++" with show;"

instance (Show prod_name, Show nonTerm, Show term, RegExp term) => View2Abstract (Prod prod_name nonTerm term) where
  view2Abstract prod@(Prod prod_name lhsNonTerm rhs) =
      let s1               = "abstract production p_" ++ show prod_name
          s2               = "this::" ++ show lhsNonTerm ++ " ::= " ++ (intercalate " " $ map auxRHS (zip ['a'..'z'] rhs))
          attr_show        = attributeShow prod
      in  unlines [s1,s2,"{",attr_show,"}"]
      where
          auxRHS (c, NonTerm x) = [c]++"::"++show x
          auxRHS (c, Term    x) = ""
          auxRHS (c, Prim    x) = [c]++"::"++literal2Type x

-- -- Attribute Show
attributeShow :: (RegExp term, Show prod_name) => (Prod prod_name nonTerm term) -> String
attributeShow prod@(Prod prod_name lhsNonTerm rhs) =
      let s1 = intercalate " ++ \",\" ++ " . filter (/= "") $ map aux (zip ['a'..'z'] rhs)
      in  "  this.show = \""++show prod_name++"(\" ++ "++ s1 ++" ++ \")\";"
      where
          aux (c, NonTerm x) = [c]++".show"
          aux (c, Term    x) = ""
          aux (c, Prim    x) = [c]++".getInfo"
