grammar Gram;

synthesized attribute toConcreteE :: E;
synthesized attribute toConcreteT :: T;
synthesized attribute toConcreteF :: F;

nonterminal E_A with pp, show, toConcreteE;
nonterminal T_A with pp, show, toConcreteT;
nonterminal F_A with pp, show, toConcreteF;

abstract production p_Add_A
this::E_A ::= a::E_A c::T_A
{
  this.show = "Add(" ++ a.show ++ "," ++ c.show ++ ")";
  this.pp = a.pp ++ " " ++ c.pp;
  this.toConcreteE = p_Add (a.toConcreteE, terminal (Plus, "+"), c.toConcreteT);
}

abstract production p_Sub_A
this::E_A ::= a::E_A c::T_A
{
  this.show = "Sub(" ++ a.show ++ "," ++ c.show ++ ")";
  this.pp = a.pp ++ " " ++ " " ++ c.pp;
  this.toConcreteE = p_Sub (a.toConcreteE, terminal (Minus, "-"), c.toConcreteT);
}

abstract production p_Et_A
this::E_A ::= a::T_A
{
  this.show = "Et(" ++ a.show ++ ")";
  this.pp = a.pp;
  this.toConcreteE = p_Et(a.toConcreteT);
}

abstract production p_Mul_A
this::T_A ::= a::T_A c::F_A
{
  this.show = "Mul(" ++ a.show ++ "," ++ c.show ++ ")";
  this.pp = a.pp ++ " " ++ c.pp;
  this.toConcreteT = p_Mul(a.toConcreteT, terminal (Ex, "*"), c.toConcreteF);
}

abstract production p_Tf_A
this::T_A ::= a::F_A
{
  this.show = "Tf(" ++ a.show ++ ")";
  this.pp = a.pp;
  this.toConcreteT = p_Tf(a.toConcreteF);  
}

abstract production p_Neg_A
this::F_A ::= b::F_A
{
  this.show = "Neg(" ++ b.show ++ ")";
  this.pp = b.pp;
  this.toConcreteF = p_Neg(terminal (Minus, "-"), b.toConcreteF);  
}

abstract production p_Nest_A
this::F_A ::= b::E_A
{
  this.show = "Nest(" ++ b.show ++ ")";
  this.pp = b.pp;
  this.toConcreteF = p_Nest(terminal (LeftParen, "("), b.toConcreteE, terminal (RightParen, ")"));
}

abstract production p_Value_A
this::F_A ::= a::Int_NT
{
  this.show = "Value( " ++ a.show ++ ")";
  this.pp = a.pp;
  this.toConcreteF = p_Value(terminal (Int_Lit, a.getInfo));
}

-- These NT used to be the Literal Terminals

synthesized attribute getInfo :: String;
nonterminal Int_NT with getInfo, pp, show;

abstract production int_NT
this::Int_NT ::= v::String
{
  this.show = "int_NT (" ++ v ++ ")";
  this.pp = v;
  this.getInfo = v;
}



