grammar Gram;

synthesized attribute toAbstractE_A :: E_A;
synthesized attribute toAbstractT_A :: T_A;
synthesized attribute toAbstractF_A :: F_A;

nonterminal E with pp, show, toAbstractE_A;
nonterminal T with pp, show, toAbstractT_A;
nonterminal F with pp, show, toAbstractF_A;

terminal LeftParen '(';
terminal RightParen ')';
terminal Plus '+';
terminal Minus '-';
terminal Ex '*';


concrete production p_Add
this::E ::= a::E b::Plus c::T
{
  this.show = "Add(" ++ a.show ++ "," ++ " + " ++ "," ++ c.show ++ ")";
  this.pp = a.pp ++ " " ++ "+" ++ " " ++ c.pp;
  this.toAbstractE_A = p_Add_A (a.toAbstractE_A, c.toAbstractT_A);
}

concrete production p_Sub
this::E ::= a::E b::Minus c::T
{
  this.show = "Sub(" ++ a.show ++ "," ++ " - " ++ "," ++ c.show ++ ")";
  this.pp = a.pp ++ " " ++ "-" ++ " " ++ c.pp;
  this.toAbstractE_A = p_Sub_A (a.toAbstractE_A, c.toAbstractT_A);
}

concrete production p_Et
this::E ::= a::T
{
  this.show = "Et(" ++ a.show ++ ")";
  this.pp = a.pp;
  this.toAbstractE_A = p_Et_A(a.toAbstractT_A);
}

concrete production p_Mul
this::T ::= a::T b::Ex c::F
{
  this.show = "Mul(" ++ a.show ++ "," ++ " * " ++ "," ++ c.show ++ ")";
  this.pp = a.pp ++ " " ++ "*" ++ " " ++ c.pp;
  this.toAbstractT_A = p_Mul_A(a.toAbstractT_A, c.toAbstractF_A);
}

concrete production p_Tf
this::T ::= a::F
{
  this.show = "Tf(" ++ a.show ++ ")";
  this.pp = a.pp;
  this.toAbstractT_A = p_Tf_A(a.toAbstractF_A);
}

concrete production p_Neg
this::F ::= a::Minus b::F
{
  this.show = "Neg(" ++ " - " ++ "," ++ b.show ++ ")";
  this.pp = "-" ++ " " ++ b.pp;
  this.toAbstractF_A = p_Neg_A(b.toAbstractF_A);  
}

concrete production p_Nest
this::F ::= a::LeftParen b::E c::RightParen
{
  this.show = "Nest(" ++ " ( " ++ "," ++ b.show ++ "," ++ " ) " ++ ")";
  this.pp = "(" ++ " " ++ b.pp ++ " " ++ ")";
  this.toAbstractF_A = p_Nest_A( b.toAbstractE_A );
}

concrete production p_Value
this::F ::= a::Int_Lit
{
  this.show = "Value(" ++ a.lexeme ++ ")";
  this.pp = a.lexeme;
  this.toAbstractF_A = p_Value_A( int_NT (a.lexeme) );
}




