module MainSL where

import Bidi
import Data.List
import Data.Maybe
import Data.Data
import Gram2Silver
import GetTransf2Silver
import PutTransf2Silver
import Terminals2Silver

r :: IO()
r = do putStrLn  "\n\n***** Generating Silver Files...          *****"
       writeFile "Gram/SL_generated.sv"  (toSilver sl_grammar)
       writeFile "Gram/Get_Generated.sv" (toSilverGet getSL2)
       writeFile "Gram/Put_Generated.sv" (toSilverPut $ invert getSL2)	   
       writeFile "Gram/Terminals.sv"     (initialize) 
       putStrLn  "***** Silver Files generated in './Gram'. *****\n"


-- -- -- SimpleLanguage Grammar

data SL_Terminals = LeftSquareBrackets | RightSquareBrackets | Equal | Semicollon | Plus | Minus | Times | LeftParen | RightParen | INT | STR | IF | THEN | ELSE | PRINT | WHILE | REPEAT | UNTIL | VerticalBar | LeftBracket | RightBracket | ATTRIB
                  deriving (Show, Eq, Ord)

instance RegExp SL_Terminals where
    regExp LeftSquareBrackets  = "["
    regExp RightSquareBrackets = "]"
    regExp Equal               = "="
    regExp Semicollon          = ";"
    regExp Plus                = "+"
    regExp Minus               = "-"
    regExp Times               = "x"
    regExp LeftParen           = "("
    regExp RightParen          = ")"
    regExp INT                 = "INT"
    regExp STR                 = "STR"
    regExp IF                  = "IF"
    regExp THEN                = "THEN"
    regExp ELSE                = "ELSE"
    regExp PRINT               = "PRINT"
    regExp WHILE               = "WHILE"
    regExp REPEAT              = "REPEAT"
    regExp UNTIL               = "UNTIL"
    regExp VerticalBar         = "|"
    regExp LeftBracket         = "{"
    regExp RightBracket        = "}"
    regExp ATTRIB              = "ATTRIB"

data SL_Nonterminals = SL | DeclsList | Type | Stat | Expr | ListStat | Txpr | Fxpr
                     deriving (Show, Eq, Ord)

data SL_ProdNames = Begin | OneDecl | TipoInt | TipoString | VariousDecl | OneStat | VariousStat | Attrib | Sum | Sub | Et | Mul | Tf | Neg | Nest | Const | Variable | IfElse | Print | While | RepeatUntil
                  deriving (Show, Eq, Ord)
		
sl_grammar :: CFG SL_Terminals SL_Nonterminals SL_ProdNames
sl_grammar =  CFG
-- NonTerms and Prims
  [SL, DeclsList, Type, Stat, ListStat, Expr, Txpr, Fxpr] [String_Lit, Int_Lit]
-- Terms
  [LeftSquareBrackets, RightSquareBrackets, Equal, Semicollon, Plus, Minus, Times, LeftParen, RightParen, INT, STR, IF, THEN, ELSE, PRINT, WHILE, REPEAT, UNTIL, VerticalBar, LeftBracket, RightBracket, ATTRIB]
-- Prod Names
  [Begin, OneDecl, TipoInt, TipoString, VariousDecl, OneStat, VariousStat, Attrib, Const, Variable, Sum, Sub, Et, Mul, Tf, Neg, Nest, Const, Variable, IfElse, Print, While, RepeatUntil]
-- Prods
  [Prod Begin       SL        [Term LeftSquareBrackets, NonTerm DeclsList, Term VerticalBar, NonTerm ListStat, Term RightSquareBrackets],

   Prod OneDecl     DeclsList [NonTerm Type, Prim String_Lit, Term Semicollon],
   Prod VariousDecl DeclsList [NonTerm Type, Prim String_Lit, Term Semicollon, NonTerm DeclsList],

   Prod TipoInt     Type      [Term INT],
   Prod TipoString  Type      [Term STR],

   Prod OneStat     ListStat  [NonTerm Stat],
   Prod VariousStat ListStat  [NonTerm Stat, NonTerm ListStat],

   Prod Attrib      Stat      [Term ATTRIB, Prim String_Lit, Term Equal, NonTerm Expr, Term Semicollon],
--   Prod IfElse      Stat      [Term IF, NonTerm Expr, Term THEN, Term LeftBracket, NonTerm ListStat, Term RightBracket, Term ELSE, Term LeftBracket, NonTerm ListStat, Term RightBracket],
   Prod While       Stat      [Term WHILE, NonTerm Expr, Term LeftBracket, NonTerm ListStat, Term RightBracket],
   Prod Print       Stat      [Term PRINT, NonTerm Expr, Term Semicollon],
   Prod RepeatUntil Stat      [Term REPEAT, Term LeftBracket, NonTerm ListStat, Term RightBracket, Term UNTIL, NonTerm Expr],

   Prod Sum         Expr      [NonTerm Expr, Term Plus,  NonTerm Txpr],
   Prod Sub         Expr      [NonTerm Expr, Term Minus, NonTerm Txpr],
   Prod Et          Expr      [NonTerm Txpr],
   Prod Mul         Txpr      [NonTerm Txpr, Term Times, NonTerm Fxpr],
   Prod Tf          Txpr      [NonTerm Fxpr],
   Prod Neg         Fxpr      [Term Minus, NonTerm Fxpr],
   Prod Nest        Fxpr      [Term LeftParen, NonTerm Expr, Term RightParen],
   Prod Const       Fxpr      [Prim Int_Lit],
   Prod Variable    Fxpr      [Prim String_Lit]
  ]

-- -- -- Transformation from SL to SL
ntSL2 :: NTFunc SL_Nonterminals SL_Nonterminals
ntSL2 = [(SL,[SL]),(DeclsList,[DeclsList]),(Type,[Type]),(ListStat,[ListStat]),(Stat,[Stat]),(Expr,[Expr]),(Txpr,[Txpr]),(Fxpr,[Fxpr])]

rulesSL2 :: [Rule SL_Terminals SL_Nonterminals SL_ProdNames 
                 SL_Terminals SL_Nonterminals SL_ProdNames]
rulesSL2 =
 [  ( Pattern (Begin,SL)              [P_Term LeftSquareBrackets, P_Arg('a',DeclsList), P_Term VerticalBar, P_Arg('b',ListStat), P_Term RightSquareBrackets],
      Pattern (Begin,SL)              [P_Term LeftSquareBrackets, P_Arg('a',DeclsList), P_Term VerticalBar, P_Arg('b',ListStat), P_Term RightSquareBrackets] )

  , ( Pattern (OneDecl,DeclsList)     [P_Arg('a',Type), P_Prim ('i',String_Lit), P_Term Semicollon],
      Pattern (OneDecl,DeclsList)     [P_Arg('a',Type), P_Prim ('i',String_Lit), P_Term Semicollon])
  , ( Pattern (VariousDecl,DeclsList) [P_Arg('a',Type), P_Prim ('i',String_Lit), P_Term Semicollon, P_Arg('b',DeclsList)],
      Pattern (VariousDecl,DeclsList) [P_Arg('a',Type), P_Prim ('i',String_Lit), P_Term Semicollon, P_Arg('b',DeclsList)])
 
  , ( Pattern (TipoInt,Type)          [P_Term INT],  Pattern (TipoInt,Type)          [P_Term INT])
  , ( Pattern (TipoString,Type)       [P_Term STR],  Pattern (TipoString,Type)       [P_Term STR])

  , ( Pattern (OneStat,ListStat)      [P_Arg('a',Stat)],                     Pattern (OneStat, ListStat)     [P_Arg('a',Stat)])
  , ( Pattern (VariousStat,ListStat)  [P_Arg('a',Stat),P_Arg('b',ListStat)], Pattern (VariousStat, ListStat) [P_Arg('a',Stat),P_Arg('b',ListStat)])

  , ( Pattern (Attrib,Stat)           [P_Term ATTRIB, P_Prim ('i',String_Lit), P_Term Equal, P_Arg('a',Expr), P_Term Semicollon],
      Pattern (Attrib,Stat)           [P_Term ATTRIB, P_Prim ('i',String_Lit), P_Term Equal, P_Arg('a',Expr), P_Term Semicollon])
--  , ( Pattern (IfElse,Stat)           [P_Term IF, P_Arg('a',Expr), P_Term THEN, P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket, P_Term ELSE, P_Term LeftBracket, P_Arg('c',ListStat), P_Term RightBracket],
--      Pattern (IfElse,Stat)           [P_Term IF, P_Arg('a',Expr), P_Term THEN, P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket, P_Term ELSE, P_Term LeftBracket, P_Arg('c',ListStat), P_Term RightBracket])
  , ( Pattern (While,Stat)            [P_Term WHILE, P_Arg('a',Expr), P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket],
      Pattern (While,Stat)            [P_Term WHILE, P_Arg('a',Expr), P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket])
  , ( Pattern (Print,Stat)            [P_Term PRINT, P_Arg('a',Expr), P_Term Semicollon],
      Pattern (Print,Stat)            [P_Term PRINT, P_Arg('a',Expr), P_Term Semicollon])

  , ( Pattern (RepeatUntil,Stat)      [P_Term REPEAT, P_Term LeftBracket, P_Arg('a',ListStat), P_Term RightBracket, P_Term UNTIL, P_Arg('b',Expr)],
      Pattern (While,Stat)            [P_Term WHILE, P_Arg('b',Expr), P_Term LeftBracket, P_Arg('a',ListStat), P_Term RightBracket ])

  --, ( Pattern (Sub,Expr)              [Pattern (Et,Expr) [Pattern (Tf,Txpr) [Pattern (Const,Fxpr) [P_Lexemme ('i',"0",Int_Lit)]]], P_Term Minus, P_Arg ('a',Txpr)],
  --    Pattern (Et,Expr)               [Pattern (Tf,Txpr) [Pattern (Neg,Fxpr) [P_Term Minus, Pattern (Nest,Fxpr) [P_Term LeftParen, Pattern (Et,Expr) [P_Arg ('a',Txpr)], P_Term RightParen]]]]
  --  )
  , ( Pattern (Sum,Expr)              [P_Arg ('a',Expr), P_Term Plus,  P_Arg ('b',Txpr)], Pattern (Sum,Expr)              [P_Arg ('a',Expr), P_Term Plus,  P_Arg ('b',Txpr)])
  , ( Pattern (Sub,Expr)              [P_Arg ('a',Expr), P_Term Minus, P_Arg ('b',Txpr)], Pattern (Sub,Expr)              [P_Arg ('a',Expr), P_Term Minus, P_Arg ('b',Txpr)])
  , ( Pattern (Et,Expr)               [P_Arg ('a',Txpr)],                                 Pattern (Et,Expr)               [P_Arg ('a',Txpr)] )
  , ( Pattern (Mul,Txpr)              [P_Arg ('a',Txpr), P_Term Times, P_Arg ('b',Fxpr)], Pattern (Mul,Txpr)              [P_Arg ('a',Txpr), P_Term Times, P_Arg ('b',Fxpr)])
  , ( Pattern (Tf,Txpr)               [P_Arg ('a',Fxpr)]                                , Pattern (Tf,Txpr)               [P_Arg ('a',Fxpr)])
  , ( Pattern (Neg,Fxpr)              [P_Term Minus, P_Arg ('a',Fxpr)]                  , Pattern (Neg,Fxpr)              [P_Term Minus, P_Arg ('a',Fxpr)])
  , ( Pattern (Nest,Fxpr)             [P_Term LeftParen, P_Arg ('a',Expr), P_Term RightParen], Pattern (Nest,Fxpr) [P_Term LeftParen, P_Arg ('a',Expr), P_Term RightParen])
  , ( Pattern (Const,Fxpr)            [P_Prim ('a',Int_Lit)],    Pattern (Const,Fxpr)            [P_Prim ('a',Int_Lit)])
  , ( Pattern (Variable,Fxpr)         [P_Prim ('a',String_Lit)], Pattern (Variable,Fxpr)         [P_Prim ('a',String_Lit)])
 ]

getSL2 :: Transf SL_Terminals SL_Nonterminals SL_ProdNames
                SL_Terminals SL_Nonterminals SL_ProdNames
getSL2 =  Transf ntSL2 rulesSL2



















































-- -- -- Transformation from SL to SL
-- This transformation corresponds to the identity
ntSL :: NTFunc SL_Nonterminals SL_Nonterminals
ntSL = [(SL,[SL]),(DeclsList,[DeclsList]),(Type,[Type]),(ListStat,[ListStat]),(Stat,[Stat]),(Expr,[Expr]),(Txpr,[Txpr]),(Fxpr,[Fxpr])]

rulesSL :: [Rule SL_Terminals SL_Nonterminals SL_ProdNames 
                 SL_Terminals SL_Nonterminals SL_ProdNames]
rulesSL =
 [  ( Pattern (Begin,SL)              [P_Term LeftSquareBrackets, P_Arg('a',DeclsList), P_Term VerticalBar, P_Arg('b',ListStat), P_Term RightSquareBrackets],
      Pattern (Begin,SL)              [P_Term LeftSquareBrackets, P_Arg('a',DeclsList), P_Term VerticalBar, P_Arg('b',ListStat), P_Term RightSquareBrackets] )

  , ( Pattern (OneDecl,DeclsList)     [P_Arg('a',Type), P_Prim ('i',String_Lit), P_Term Semicollon],
      Pattern (OneDecl,DeclsList)     [P_Arg('a',Type), P_Prim ('i',String_Lit), P_Term Semicollon])
  , ( Pattern (VariousDecl,DeclsList) [P_Arg('a',Type), P_Prim ('i',String_Lit), P_Term Semicollon, P_Arg('b',DeclsList)],
      Pattern (VariousDecl,DeclsList) [P_Arg('a',Type), P_Prim ('i',String_Lit), P_Term Semicollon, P_Arg('b',DeclsList)])
 
  , ( Pattern (TipoInt,Type)          [P_Term INT],  Pattern (TipoInt,Type)          [P_Term INT])
  , ( Pattern (TipoString,Type)       [P_Term STR],  Pattern (TipoString,Type)       [P_Term STR])

  , ( Pattern (OneStat,ListStat)      [P_Arg('a',Stat)],                     Pattern (OneStat, ListStat)     [P_Arg('a',Stat)])
  , ( Pattern (VariousStat,ListStat)  [P_Arg('a',Stat),P_Arg('b',ListStat)], Pattern (VariousStat, ListStat) [P_Arg('a',Stat),P_Arg('b',ListStat)])

  , ( Pattern (Attrib,Stat)           [P_Term ATTRIB, P_Prim ('i',String_Lit), P_Term Equal, P_Arg('a',Expr), P_Term Semicollon],
      Pattern (Attrib,Stat)           [P_Term ATTRIB, P_Prim ('i',String_Lit), P_Term Equal, P_Arg('a',Expr), P_Term Semicollon])
--  , ( Pattern (IfElse,Stat)           [P_Term IF, P_Arg('a',Expr), P_Term THEN, P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket, P_Term ELSE, P_Term LeftBracket, P_Arg('c',ListStat), P_Term RightBracket],
--      Pattern (IfElse,Stat)           [P_Term IF, P_Arg('a',Expr), P_Term THEN, P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket, P_Term ELSE, P_Term LeftBracket, P_Arg('c',ListStat), P_Term RightBracket])
  , ( Pattern (While,Stat)            [P_Term WHILE, P_Arg('a',Expr), P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket],
      Pattern (While,Stat)            [P_Term WHILE, P_Arg('a',Expr), P_Term LeftBracket, P_Arg('b',ListStat), P_Term RightBracket])
  , ( Pattern (Print,Stat)            [P_Term PRINT, P_Arg('a',Expr), P_Term Semicollon],
      Pattern (Print,Stat)            [P_Term PRINT, P_Arg('a',Expr), P_Term Semicollon])
  , ( Pattern (RepeatUntil,Stat)      [P_Term REPEAT, P_Term LeftBracket, P_Arg('a',ListStat), P_Term RightBracket, P_Term UNTIL, P_Arg('b',Expr)],
      Pattern (RepeatUntil,Stat)      [P_Term REPEAT, P_Term LeftBracket, P_Arg('a',ListStat), P_Term RightBracket, P_Term UNTIL, P_Arg('b',Expr)])

  , ( Pattern (Sum,Expr)              [P_Arg ('a',Expr), P_Term Plus,  P_Arg ('b',Txpr)], Pattern (Sum,Expr)              [P_Arg ('a',Expr), P_Term Plus,  P_Arg ('b',Txpr)])
  , ( Pattern (Sub,Expr)              [P_Arg ('a',Expr), P_Term Minus, P_Arg ('b',Txpr)], Pattern (Sub,Expr)              [P_Arg ('a',Expr), P_Term Minus, P_Arg ('b',Txpr)])
  , ( Pattern (Et,Expr)               [P_Arg ('a',Txpr)],                                 Pattern (Et,Expr)               [P_Arg ('a',Txpr)] )
  , ( Pattern (Mul,Txpr)              [P_Arg ('a',Txpr), P_Term Times, P_Arg ('b',Fxpr)], Pattern (Mul,Txpr)              [P_Arg ('a',Txpr), P_Term Times, P_Arg ('b',Fxpr)])
  , ( Pattern (Tf,Txpr)               [P_Arg ('a',Fxpr)]                                , Pattern (Tf,Txpr)               [P_Arg ('a',Fxpr)])
  , ( Pattern (Neg,Fxpr)              [P_Term Minus, P_Arg ('a',Fxpr)]                  , Pattern (Neg,Fxpr)              [P_Term Minus, P_Arg ('a',Fxpr)])
  , ( Pattern (Nest,Fxpr)             [P_Term LeftParen, P_Arg ('a',Expr), P_Term RightParen], Pattern (Nest,Fxpr) [P_Term LeftParen, P_Arg ('a',Expr), P_Term RightParen])
  , ( Pattern (Const,Fxpr)            [P_Prim ('a',Int_Lit)],    Pattern (Const,Fxpr)            [P_Prim ('a',Int_Lit)])
  , ( Pattern (Variable,Fxpr)         [P_Prim ('a',String_Lit)], Pattern (Variable,Fxpr)         [P_Prim ('a',String_Lit)])
 ]

getSL :: Transf SL_Terminals SL_Nonterminals SL_ProdNames
                SL_Terminals SL_Nonterminals SL_ProdNames
getSL =  Transf ntSL rulesSL

