grammar Gram;


nonterminal SL with pp, show;
nonterminal DeclsList with pp, show;
nonterminal Type with pp, show;
nonterminal Stat with pp, show;
nonterminal ListStat with pp, show;
nonterminal Expr with pp, show;
nonterminal Txpr with pp, show;
nonterminal Fxpr with pp, show;


terminal LeftSquareBrackets '[';
terminal RightSquareBrackets ']';
terminal Equal '=';
terminal Semicollon ';';
terminal Plus '+';
terminal Minus '-';
terminal Times 'x';
terminal LeftParen '(';
terminal RightParen ')';
terminal INT 'INT';
terminal STR 'STR';
terminal IF 'IF';
terminal THEN 'THEN';
terminal ELSE 'ELSE';
terminal PRINT 'PRINT';
terminal WHILE 'WHILE';
terminal REPEAT 'REPEAT';
terminal UNTIL 'UNTIL';
terminal VerticalBar '|';
terminal LeftBracket '{';
terminal RightBracket '}';


concrete production p_Begin
this::SL ::= a::LeftSquareBrackets b::DeclsList c::VerticalBar d::ListStat e::RightSquareBrackets
{
  this.show = "Begin(" ++ " [ " ++ "," ++ b.show ++ "," ++ " | " ++ "," ++ d.show ++ "," ++ " ] " ++ ")";
}

concrete production p_OneDecl
this::DeclsList ::= a::Type b::String_Lit c::Semicollon
{
  this.show = "OneDecl(" ++ a.show ++ "," ++ b.lexeme ++ "," ++ " ; " ++ ")";
}

concrete production p_VariousDecl
this::DeclsList ::= a::Type b::String_Lit c::Semicollon d::DeclsList
{
  this.show = "VariousDecl(" ++ a.show ++ "," ++ b.lexeme ++ "," ++ " ; " ++ "," ++ d.show ++ ")";
}

concrete production p_TipoInt
this::Type ::= a::INT
{
  this.show = "TipoInt(" ++ " INT " ++ ")";
}

concrete production p_TipoString
this::Type ::= a::STR
{
  this.show = "TipoString(" ++ " STR " ++ ")";
}

concrete production p_OneStat
this::ListStat ::= a::Stat
{
  this.show = "OneStat(" ++ a.show ++ ")";
}

concrete production p_VariousStat
this::ListStat ::= a::Stat b::ListStat
{
  this.show = "VariousStat(" ++ a.show ++ "," ++ b.show ++ ")";
}

concrete production p_IfElse
this::Stat ::= a::IF b::LeftParen c::Expr d::RightParen e::THEN f::LeftBracket g::ListStat h::RightBracket i::ELSE j::LeftBracket k::ListStat l::RightBracket
{
  this.show = "IfElse(" ++ " IF " ++ "," ++ " ( " ++ "," ++ c.show ++ "," ++ " ) " ++ "," ++ " THEN " ++ "," ++ " { " ++ "," ++ g.show ++ "," ++ " } " ++ "," ++ " ELSE " ++ "," ++ " { " ++ "," ++ k.show ++ "," ++ " } " ++ ")";
}

concrete production p_While
this::Stat ::= a::WHILE b::LeftParen c::Expr d::RightParen e::LeftBracket f::ListStat g::RightBracket
{
  this.show = "While(" ++ " WHILE " ++ "," ++ " ( " ++ "," ++ c.show ++ "," ++ " ) " ++ "," ++ " { " ++ "," ++ f.show ++ "," ++ " } " ++ ")";
}

concrete production p_Print
this::Stat ::= a::PRINT b::Expr c::Semicollon
{
  this.show = "Print(" ++ " PRINT " ++ "," ++ b.show ++ "," ++ " ; " ++ ")";
}

concrete production p_RepeatUntil
this::Stat ::= a::REPEAT b::LeftBracket c::ListStat d::RightBracket e::UNTIL f::Expr
{
  this.show = "RepeatUntil(" ++ " REPEAT " ++ "," ++ " { " ++ "," ++ c.show ++ "," ++ " } " ++ "," ++ " UNTIL " ++ "," ++ f.show ++ ")";
}

concrete production p_Sum
this::Expr ::= a::Expr b::Plus c::Txpr
{
  this.show = "Sum(" ++ a.show ++ "," ++ " + " ++ "," ++ c.show ++ ")";
}

concrete production p_Sub
this::Expr ::= a::Expr b::Minus c::Txpr
{
  this.show = "Sub(" ++ a.show ++ "," ++ " - " ++ "," ++ c.show ++ ")";
}

concrete production p_Et
this::Expr ::= a::Txpr
{
  this.show = "Et(" ++ a.show ++ ")";
}

concrete production p_Mul
this::Txpr ::= a::Txpr b::Times c::Fxpr
{
  this.show = "Mul(" ++ a.show ++ "," ++ " x " ++ "," ++ c.show ++ ")";
}

concrete production p_Tf
this::Txpr ::= a::Fxpr
{
  this.show = "Tf(" ++ a.show ++ ")";
}

concrete production p_Neg
this::Fxpr ::= a::Minus b::Fxpr
{
  this.show = "Neg(" ++ " - " ++ "," ++ b.show ++ ")";
}

concrete production p_Nest
this::Fxpr ::= a::LeftParen b::Expr c::RightParen
{
  this.show = "Nest(" ++ " ( " ++ "," ++ b.show ++ "," ++ " ) " ++ ")";
}

concrete production p_Const
this::Fxpr ::= a::Int_Lit
{
  this.show = "Const(" ++ a.lexeme ++ ")";
}

concrete production p_Variable
this::Fxpr ::= a::String_Lit
{
  this.show = "Variable(" ++ a.lexeme ++ ")";
}


