{-# LANGUAGE FlexibleInstances #-}

module Repair2Silver where

import IntermediateTransfType
import BidiEngine
import Data.List
import Types

writeAttrs :: (Show nonTermA, Show nonTermB) => Transf termA nonTermA prodNameA termB nonTermB prodNameB -> String
writeAttrs (Transf rules) = unlines . nub . map (aux . fst) $ rules
                          where aux (P_Put to patt) = let from = getPatternType patt
                                                      in  "synthesized attribute needs_repair_"++show from++"_"++show to++" :: Boolean occurs on "++show from++";"

createAttrs :: (Show nonTermA, Show nonTermB, Show prodNameB, Show prodNameA, Show termA, Show termB, RegExp termA, RegExp termB, Eq prodNameA, Eq nonTermA, Eq termA, Eq prodNameB, Eq nonTermB, Eq termB)
            => [Rule termA nonTermA prodNameA termB nonTermB prodNameB]
            -> CanBes termA nonTermA prodNameA termB nonTermB prodNameB
            -> [nonTermA]
            -> String
createAttrs l canBes listNonTerms =  let number_attrs = length l
                                         attr_name    = "needs_repair_"++from++"_"++to
                                         s1           = "  this."++attr_name++" = "++(attr_name++"_"++show number_attrs)++";"
                                         -- The zip is to know which local attribute I am creating when creating each local get
                                         s2           = intercalate "\n" . map createAttr $ zip (reverse [1..number_attrs]) l
                                         s3           = "  local attribute "++attr_name++"_0 :: Boolean = true;"
                                     in  intercalate "\n" [s1,s2,s3]
                                     where random_patt = head $ l
                                           from        = show . (\(P_Put _  patt) -> getPatternType patt) . fst $ random_patt
                                           to          = show . getPatternType . snd $ random_patt
                                           prod        = show . (\(P_Put _  patt) -> getPatternProd patt) . fst $ random_patt

createAttr :: (Show nonTermA, Show nonTermB, Show prodNameB, Show prodNameA, Show termA, Show termB, RegExp termA, RegExp termB, Eq prodNameA, Eq nonTermA, Eq termA, Eq prodNameB, Eq nonTermB, Eq termB )
           => (Int, Rule termA nonTermA prodNameA termB nonTermB prodNameB) -> String
createAttr (int, rule) = let s1    = "  local attribute needs_repair_"++from++"_"++to++"_"++show int++" :: Boolean = case this of"
                             ident = spaces ((length s1) - 14) -- 14 is to compensate the static part. Only the name of the attribute changes.
                             s2    = ident ++ "| " ++ create_cases rule from ("needs_repair_"++from++"_"++to++"_"++show (int-1))
                             s3    = ident ++ "| _ -> "++"needs_repair_"++from++"_"++to++"_"++show (int-1)
                             s4    = ident ++ "end;"
                         in  intercalate "\n" [s1, s2, s3, s4]
                         where from = show . (\(P_Put _  patt) -> getPatternType patt) . fst $ rule
                               to   = show . getPatternType . snd $ rule
                               prod = show . (\(P_Put _  patt) -> getPatternProd patt) . fst $ rule

create_cases rule from default_case = if isPatternSimple ((\(P_Put _ p) -> p) . fst $ rule) then sourcePattern2Silver (fst rule) ++ " -> false"
                                      else complex_cases rule default_case

-- A pattern being simple means it only has attribute calls to arguments, never to whole Patterns.
isPatternSimple (P_Arg   n t    ) = True
isPatternSimple (Pattern p t rhs) = True && and (map isPatternSimple rhs)
isPatternSimple (P_Lexemme n p t) = True
isPatternSimple (P_Term    p    ) = True
isPatternSimple (P_Prim  n t    ) = True
isPatternSimple (P_Put t (Pattern _ _ _) ) = False
isPatternSimple (P_Put t p      ) = True
isPatternSimple (P_Get t (Pattern _ _ _) ) = False
isPatternSimple (P_Get t   p    ) = True

-- complex_cases is the function to deal with patterns where we have attribute calls to whole arguments, as in:
-- ( get Decl_c Decl (Pattern P_multiDecl_c Decl_c [P_Arg 't' Type_c, Pattern P_consVar_c Vars_c [P_Prim 'v' String_Lit, P_Term Comma_t, P_Arg 'a' Vars_c], P_Term Semi_t ]),
-- Pattern P_seqDecl Decl [Pattern P_varDecl Decl [get Type_c Type (P_Arg 't' Type_c), P_Prim 'v' String_Lit], get Decl_c Decl (Pattern P_multiDecl_c Decl_c [P_Arg 't' Type_c,P_Arg 'a' Vars_c , P_Term Semi_t])] 
complex_cases rule default_case
  = let -- First we collect all the pattens that will become cases,  and give them variable names
        -- We give them names like "aa" and "bb" because 'a' and 'b' are already assigned
       patt_to_cases           = zip (map (\a -> "aa") ['a'..'z']) (gather_patterns ((\(P_Put _ p) -> p) . fst $ rule))
       -- Secondly, we collect all the variables in the lhs of the rule with the same name.
       -- We will also need to test these.
       all_variables           = gather_variables ((\(P_Put _ p) -> p) . fst $ rule)
       -- Check which variables appear more than once, which is basically the difference between the original list and the list of unique elements
       variablesWithRepetition = nub $ all_variables \\ (nub all_variables)
       -- Now we convert the pattern to Silver cases, and we change recursive called patterns to their variable names
       start      = complexSourcePattern2Silver patt_to_cases ((\(P_Put _ p) -> p) . fst $ rule) ++ " -> case "++intercalate ", " (map helper patt_to_cases)++" of"
       -- This is an auxiliary step for complexCases, because on the if comparison I will need to ask for certain attribute I will need to know their types
       variablesWithRepetitionAndType = concat $ map (searchVarType ((\(P_Put n p) -> p) . fst $ rule)) variablesWithRepetition
       cases      = concat $ map (complexCases (snd rule) default_case variablesWithRepetitionAndType) patt_to_cases
       last_case  = spaces 90 ++ "| _ -> "++default_case
       end        = spaces 90 ++ "end"
    in intercalate "\n" [start, cases,last_case,end]
    where helper (a, (P_Get t p)) = a++".put_"++show t++"_"++show (getPatternType p)

complexCases final_patt default_case variablesWithRepetitionAndType (var, P_Get t p) = let s1 = spaces 90 ++ "| " ++ secondComplexSourcePattern2Silver variablesWithRepetitionAndType p ++ " -> "
                                                                                           -- I need this to know in which variables I need to call an attribute
                                                                                           aux_vars = (gather_variables p) \\ (map fst variablesWithRepetitionAndType)
                                                                                           s2 = "if ("++(intercalate "==" $ map aux variablesWithRepetitionAndType)++") then false else "++default_case
                                                                                       in  concat [s1,s2]
                                                                                       where aux (a,b) = aux2 b++".show == "++[a]++"2.show" 
                                                                                             aux2 (P_Get   n p    ) = targetPattern2Silver_aux p++".put_"++show n++"_"++show (getPatternType p)


searchVarType (P_Arg   n t    )        var = []
searchVarType (Pattern p t rhs)        var = concat $ map (\x -> searchVarType x var) rhs
searchVarType (P_Lexemme n p t)        var = []
searchVarType (P_Term    p    )        var = []
searchVarType (P_Prim  n t    )        var = []
searchVarType p@(P_Get t (P_Arg n _ )) var = if (n==var) then [(var,p)] else []
searchVarType p@(P_Put t (P_Arg n _ )) var = if (n==var) then [(var,p)] else []
searchVarType _                        _   = []

secondComplexSourcePattern2Silver variablesWithRepetition (P_Arg   n t)     = if elem n (map fst variablesWithRepetition) then ([n]++"2") else [n]
secondComplexSourcePattern2Silver variablesWithRepetition (Pattern p t rhs) = "p_"++(show p)++"( "++(intercalate ", " $ map (secondComplexSourcePattern2Silver variablesWithRepetition ) rhs)++" )"
secondComplexSourcePattern2Silver variablesWithRepetition (P_Lexemme n p t) = [n]
secondComplexSourcePattern2Silver variablesWithRepetition (P_Term    p    ) = "_"
secondComplexSourcePattern2Silver variablesWithRepetition (P_Prim  n t    ) = [n]

complexSourcePattern2Silver list (P_Arg   n t    ) = [n]
complexSourcePattern2Silver list (Pattern p t rhs) = "p_"++(show p)++"( "++(intercalate ", " $ map (complexSourcePattern2Silver list) rhs)++" )"
complexSourcePattern2Silver list (P_Lexemme n p t) = [n]
complexSourcePattern2Silver list (P_Term    p    ) = "_"
complexSourcePattern2Silver list (P_Prim  n t    ) = [n]
complexSourcePattern2Silver list p@(P_Put t patt ) = if (length (isPattern p list) > 0) then isPattern p list else aux_complexSourcePattern2Silver list patt
complexSourcePattern2Silver list p@(P_Get t patt ) = if (length (isPattern p list) > 0) then isPattern p list else aux_complexSourcePattern2Silver list patt
isPattern patt []            = []
isPattern patt ((var, p):xs) = if (patt==p) then var else isPattern patt xs
aux_complexSourcePattern2Silver list (P_Arg   n t    ) = [n]
aux_complexSourcePattern2Silver list (Pattern p t rhs) = "p_"++(show p)++"( "++(intercalate ", " $ map (aux_complexSourcePattern2Silver list) rhs)++" )"
aux_complexSourcePattern2Silver list (P_Lexemme n p t) = [n]
aux_complexSourcePattern2Silver list (P_Term    p    ) = "_"
aux_complexSourcePattern2Silver list (P_Prim  n t    ) = [n]
aux_complexSourcePattern2Silver list p@(P_Put t patt ) = complexSourcePattern2Silver list patt
aux_complexSourcePattern2Silver list p@(P_Get t patt ) = complexSourcePattern2Silver list patt

gather_patterns (P_Arg   n t    ) = []
gather_patterns (Pattern p t rhs) = concat (map gather_patterns rhs)
gather_patterns (P_Lexemme n p t) = []
gather_patterns (P_Term    p    ) = []
gather_patterns (P_Prim  n t    ) = []
gather_patterns p@(P_Put t (Pattern _ _ _) ) = [p]
gather_patterns (P_Put t p      ) = []
gather_patterns p@(P_Get t (Pattern _ _ _) ) = [p]
gather_patterns (P_Get t   p    ) = []

gather_variables (P_Arg   n t    ) = [n]
gather_variables (Pattern p t rhs) = concat (map gather_variables rhs)
gather_variables (P_Lexemme n p t) = []
gather_variables (P_Term    p    ) = []
gather_variables (P_Prim  n t    ) = [n]
gather_variables (P_Put t   p    ) = aux_gather_variables p
gather_variables (P_Get t   p    ) = aux_gather_variables p
aux_gather_variables (P_Arg   n t    ) = [n]
aux_gather_variables (Pattern p t rhs) = concat (map aux_gather_variables rhs)
aux_gather_variables (P_Lexemme n p t) = []
aux_gather_variables (P_Term    p    ) = []
aux_gather_variables (P_Prim  n t    ) = [n]
aux_gather_variables (P_Put t   p    ) = gather_variables p
aux_gather_variables (P_Get t   p    ) = gather_variables p









sourcePattern2Silver :: (Show nonTermA, Show nonTermB, Show prodNameB, Show prodNameA, Show termA, Show termB, RegExp termA, RegExp termB)
                     => (Pattern termA nonTermA prodNameA termB nonTermB prodNameB) -> String
sourcePattern2Silver (P_Arg   n t    ) = [n]
sourcePattern2Silver (Pattern p t rhs) = "p_"++(show p)++"( "++(intercalate ", " $ map sourcePattern2Silver rhs)++" )"
sourcePattern2Silver (P_Lexemme n p t) = [n]
sourcePattern2Silver (P_Term    p    ) = "_"
sourcePattern2Silver (P_Prim  n t    ) = [n]
sourcePattern2Silver (P_Put t   p    ) = sourcePattern2Silver p
sourcePattern2Silver (P_Get t   p    ) = sourcePattern2Silver p


targetPattern2Silver :: (Show nonTermA, Show nonTermB, Show prodNameB, Show prodNameA, Show termA, Show termB, RegExp termA, RegExp termB)
                     => String -> (Pattern termA nonTermA prodNameA termB nonTermB prodNameB) -> String
targetPattern2Silver nonTermS (Pattern p t rhs) = let s1 = map targetPattern2Silver_aux rhs
                                                  in  "p_"++(show p)++"( "++intercalate ", " s1++" )" 
targetPattern2Silver nonTermS (P_Get   n p    ) = targetPattern2Silver_aux p++".put_"++show (getPatternType p)++"_"++show n

targetPattern2Silver_aux :: (Show nonTermA, Show nonTermB, Show prodNameB, Show prodNameA, Show termA, Show termB, RegExp termA, RegExp termB)
    => (Pattern termA nonTermA prodNameA termB nonTermB prodNameB) -> String
targetPattern2Silver_aux (P_Term    p    ) = "terminal ("++show p++", \""++regExp p++"\")"
targetPattern2Silver_aux (P_Prim  n t    ) = [n]
targetPattern2Silver_aux (P_Arg   n t    ) = [n]
targetPattern2Silver_aux (Pattern p t rhs) = let s1 = map targetPattern2Silver_aux rhs
                                             in  "p_"++(show p)++"( "++intercalate ", " s1++" )"
targetPattern2Silver_aux (P_Lexemme n p t) = "terminal ("++show t++", \""++p++"\")"
targetPattern2Silver_aux (P_Get   n p    ) = targetPattern2Silver_aux p++".put_"++show (getPatternType p)++"_"++show n
targetPattern2Silver_aux (P_Put   n p    ) = targetPattern2Silver_aux p++".put_"++show (getPatternType p)++"_"++show n




